package com.huaching.parking.local.parking_local_general.controller.local;

import com.alibaba.fastjson.JSON;
import com.huaching.parking.local.parking_local_general.domain.ResponseResult;
import com.huaching.parking.local.parking_local_general.domain.local.dto.response.ThsChannelMeeageDTO;
import com.huaching.parking.local.parking_local_general.enums.local.ReasonCodeEnums;
import com.huaching.parking.local.parking_local_general.intercept.local.GeneralPrintLogAOP;
import com.huaching.parking.local.parking_local_general.service.local.base.ChannelMessageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @program: parking_local_general
 * @description: 停车场信息相关接口（下行）
 * @author: lidongsheng
 * @createData: 2020-04-17 11:41
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-17 11:41
 * @updateContent: 请求controller
 * @Version: 1.0.0
 * <p>
 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Api(value = "云平台 停车场信息相关下行接口controller")
@RestController
@RequestMapping(value = "/ths/request/parkMessage")
public class ParkMessageController {

    private static final Logger logger = LoggerFactory.getLogger(ParkMessageController.class);

    @Autowired
    private ChannelMessageService channelMessageService;

    /**
     * 根据车场编号和通道编号获得通道详细信息
     *
     * @return
     */
    @ApiOperation(value = "根据车场编号和通道编号获得通道详细信接口")
    @PostMapping("/getChannelMessage")
    @GeneralPrintLogAOP(value = "根据车场编号和通道编号获得通道详细信接口", jsonSubs = {-1})
    public ResponseResult getChannelMessage(@RequestBody String parkCode, @RequestBody String channelCode) {
        logger.info("根据车场编号和通道编号获得通道详细信息 parkCode -> {} , channelCode -> {}", parkCode, channelCode);
        ThsChannelMeeageDTO thsChannelMeeageDTO = ThsChannelMeeageDTO.poConvertToDto(channelMessageService.getChannelMessage(parkCode, channelCode));
        if (thsChannelMeeageDTO != null) {
            return ResponseResult.Builder.init().setData(JSON.toJSONString(thsChannelMeeageDTO)).build();
        } else {
            logger.error("根据车场编号和通道编号未查询到通道详细信息 parkCode -> {} , channelCode -> {}", parkCode, channelCode);
            return ResponseResult.Builder.init().setFailMessage(ReasonCodeEnums.EMPTY_RESULT, "无此通道信息").build();
        }
    }
}
