package com.huaching.parking.local.parking_local_general.controller.local;

import com.huaching.parking.local.parking_local_general.domain.ResponseResult;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsPaySyncDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.response.ThsRequestChargeDTO;
import com.huaching.parking.local.parking_local_general.enums.local.ReasonCodeEnums;
import com.huaching.parking.local.parking_local_general.intercept.local.GeneralPrintLogAOP;
import com.huaching.parking.local.parking_local_general.service.local.PaymentChargesService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @program: parking_local_general
 * @description: 费用支付相关接口（下行）
 * @author: lidongsheng
 * @createData: 2020-04-22 10:27
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-22 10:27
 * @updateContent: 费用支付相关接口（下行）
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Api(value = "云平台 费用支付相关下行接口controller")
@RestController
@RequestMapping(value = "/ths/request/paymentCharges")
public class PaymentChargesController {

    private static final Logger logger = LoggerFactory.getLogger(PaymentChargesController.class);

    @Autowired
    private PaymentChargesService paymentChargesService;

    /**
     * 根据车场编号和车牌号获得费用信息
     *
     * @return
     */
    @ApiOperation(value = "根据车场编号和车牌号获得费用信息接口")
    @PostMapping("/requestCharge")
    @GeneralPrintLogAOP(value = "根据车场编号和车牌号获得费用信息接口", jsonSubs = {-1})
    public ResponseResult requestCharge(String parkCode, String carNo) {
        ThsRequestChargeDTO thsRequestChargeDTO = paymentChargesService.requestCharge(parkCode, carNo, null);
        if (thsRequestChargeDTO.getCode() != 1) {
            return ResponseResult.Builder.init().setData(thsRequestChargeDTO).build();
        } else {
            return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "请求费用失败");
        }
    }

    /**
     * 支付同步接口
     *
     * @return
     */
    @ApiOperation(value = "支付同步接口")
    @PostMapping("/paySync")
    @GeneralPrintLogAOP(value = "支付同步接口")
    public ResponseResult paySync(@RequestBody @Valid ThsPaySyncDTO thsPaySyncDTO) {

        return paymentChargesService.paySync(thsPaySyncDTO);

    }

}
