package com.huaching.parking.local.parking_local_general.controller.local;

import com.huaching.parking.local.parking_local_general.domain.ResponseResult;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsCarInDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsCarOutDTO;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsCarAdvanceInOutRecordsPO;
import com.huaching.parking.local.parking_local_general.intercept.local.GeneralPrintLogAOP;
import com.huaching.parking.local.parking_local_general.service.local.LocalCarInAndOutService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @program: parking_local_general
 * @description: 接受所有本地的（例如科拓、红门、智慧安防小区等）上行接口controller
 * @author: lidongsheng
 * @createData: 2020-04-17 11:40
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-17 11:40
 * @updateContent: 上报controller
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Api(value = "云平台 各个对接方统一上报接收controller")
@RestController
@RequestMapping(value = "/ths/report")
public class ReceiveReportController {

    private Logger logger = LoggerFactory.getLogger(ReceiveReportController.class);

    @Autowired
    private LocalCarInAndOutService localCarInAndOutService;

    /**
     * 入场记录上报
     *
     * @return
     */
    @ApiOperation(value = "各道闸入场上报接收接口")
    @PostMapping("/carIn")
    @GeneralPrintLogAOP(value = "各道闸入场上报接收接口")
    public ResponseResult carIn(@RequestBody @Valid ThsCarInDTO thsCarInDTO) {
        return localCarInAndOutService.carIn(thsCarInDTO);
    }

    /**
     * 出场记录上报
     *
     * @return
     */
    @ApiOperation(value = "各道闸出场上报接收接口")
    @PostMapping("/carOut")
    @GeneralPrintLogAOP(value = "各道闸出场上报接收接口")
    public ResponseResult carOut(@RequestBody @Valid ThsCarOutDTO thsCarOutDTO) {
        return localCarInAndOutService.carOut(thsCarOutDTO);
    }

    /**
     * 预出入场记录上报
     *
     * @return
     */
    @ApiOperation(value = "各道闸预出入场上报接收接口")
    @PostMapping("/carAdvanceInOut")
    @GeneralPrintLogAOP(value = "各道闸预出入场上报接收接口")
    public ResponseResult carAdvanceInOut(@RequestBody @Valid ThsCarAdvanceInOutRecordsPO thsCarAdvanceInOutRecordsPO) {
        return localCarInAndOutService.carAdvanceInOut(thsCarAdvanceInOutRecordsPO);
    }
}
