package com.huaching.parking.local.parking_local_general.controller.zhaf;

import com.huaching.parking.local.parking_local_general.config.zhaf.DZConfig;
import com.huaching.parking.local.parking_local_general.domain.ResponseResult;
import com.huaching.parking.local.parking_local_general.domain.zhaf.ResponseResultZHAF;
import com.huaching.parking.local.parking_local_general.domain.zhaf.dto.*;
import com.huaching.parking.local.parking_local_general.enums.local.ReasonCodeEnums;
import com.huaching.parking.local.parking_local_general.intercept.local.GeneralPrintLogAOP;
import com.huaching.parking.local.parking_local_general.service.zhaf.RemoteRequestService;
import com.huaching.parking.local.parking_local_general.util.local.DateUtils;
import com.huaching.parking.local.parking_local_general.util.zhaf.Base64Utils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ResourceUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Date;

/**
 * @program: parking_local_general
 * @description: 下行接口controller
 * @author: lidongsheng
 * @createData: 2020-04-17 11:41
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-17 11:41
 * @updateContent: 请求controller
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Api(value = "智慧安防 下行controller")
@RestController
@RequestMapping(value = "/zhaf/request")
public class RequestController {

    private static final String SAVE_IMG_ROOT_URL = "barriergateReport/";

    private static final Logger logger = LoggerFactory.getLogger(RequestController.class);

    @Autowired
    private RemoteRequestService remoteRequestService;

    /**
     * 注册设备
     *
     * @return
     */
    @ApiOperation(value = "注册设备")
    @RequestMapping(value = "/registerDevice", method = RequestMethod.POST)
    public ResponseResultZHAF registerDevice(@RequestBody RegisterDeviceDTO registerDeviceDTO) {
        return remoteRequestService.registerDevice(registerDeviceDTO);
    }

    /**
     * 订阅数据
     *
     * @return
     */
    @ApiOperation(value = "订阅数据")
    @RequestMapping(value = "/subscription", method = RequestMethod.POST)

    public ResponseResultZHAF subscription(@RequestBody SubscriptionDTO subscriptionDTO) {
        return remoteRequestService.subscription(subscriptionDTO);
    }

    /**
     * 订阅数据
     *
     * @return
     */
    @ApiOperation(value = "删除订阅数据")
    @RequestMapping(value = "/deleteSubscription", method = RequestMethod.POST)
    public ResponseResultZHAF deleteSubscription(@RequestBody SubscriptionDTO subscriptionDTO) {
        return remoteRequestService.deleteSubscription(subscriptionDTO);
    }

    /**
     * 控制道闸
     *
     * @return
     */
    @ApiOperation(value = "控制道闸")
    @RequestMapping(value = "/controlBarrierGate", method = RequestMethod.POST)
    @GeneralPrintLogAOP(value = "控制道闸", jsonSubs = {-1})
    public ResponseResult controlBarrierGate(String deviceCode, Integer type) {
        return remoteRequestService.controlBarrierGate(deviceCode, type);
    }

    /**
     * 设备关联关系新增与修改(存在则修改，不存在则新增)
     *
     * @return
     */
    @ApiOperation(value = "设备关联关系新增与修改(存在则修改，不存在则新增)")
    @RequestMapping(value = "/bindDevice", method = RequestMethod.POST)
    public ResponseResultZHAF bindDevice(@RequestBody BindDeviceDTO bindDeviceDTO) {
        return remoteRequestService.bindDevice(bindDeviceDTO);
    }

    /**
     * 删除设备
     *
     * @return
     */
    @ApiOperation(value = "删除设备")
    @RequestMapping(value = "/deleteDevice", method = RequestMethod.POST)
    public ResponseResultZHAF deleteDevice(@RequestBody DeleteDeviceDTO deleteDeviceDTO) {
        return remoteRequestService.deleteDevice(deleteDeviceDTO);
    }

    /**
     * 控制抓拍机的开闸模式
     *
     * @return
     */
    @ApiOperation(value = "控制抓拍机的开闸模式")
    @RequestMapping(value = "/ctrlMode", method = RequestMethod.POST)
    public ResponseResultZHAF ctrlMode(String deviceCode, Integer type) {
        return remoteRequestService.ctrlMode(deviceCode, type);
    }

    /**
     * base64转换为图片测试
     *
     * @return
     */
    @ApiOperation(value = "saveImgBase64")
    @RequestMapping(value = "/saveImgBase64", method = RequestMethod.POST)
    public ResponseResult saveImgBase64(@RequestBody String imageStr) {

        String date = DateUtils.dateFormatToString(new Date(), 3);
        String saveUrl = DZConfig.getSaveImgUrl() + SAVE_IMG_ROOT_URL + date;
        if (Base64Utils.GenerateImage(imageStr, saveUrl, System.currentTimeMillis() + ".jpg")) {
            return ResponseResult.responseOK();
        }
        return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "转换错误");
    }

}
