package com.huaching.parking.local.parking_local_general.domain.local.bo;

import com.alibaba.fastjson.JSON;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsChargingRulesPO;
import com.huaching.parking.local.parking_local_general.enums.local.RequestChargeTypeEnums;
import com.huaching.parking.local.parking_local_general.util.local.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.Comparator;
import java.util.Date;
import java.util.List;

/**
 * @program: parking_local_general
 * @description: 按照时间间隔计算费用
 * @author: lidongsheng
 * @createData: 2020-04-26 10:31
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-26 10:31
 * @updateContent: 按照时间间隔计算费用
 * @Version: 1.0.0
 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Component
public class ThsIntervalPayableMoney implements ThsRequestChargeAbstract {

    private static final Logger logger = LoggerFactory.getLogger(ThsIntervalPayableMoney.class);

    @Override
    public Integer getPayableMoney(ThsChargingRulesPO thsChargingRulesPO, Date inTime) {
        String chargeDetails = thsChargingRulesPO.getChargeDetails();
        return intervalPayableMoney(JSON.parseObject(chargeDetails, IntervalChargeDetail.class), inTime, thsChargingRulesPO.getFreeTime()
                , thsChargingRulesPO.getMaxOneTimeMoney());
    }

    @Override
    public Integer getType() {
        return RequestChargeTypeEnums.INTERVAL.getCode();
    }

    /**
     * 按照间隔收费
     *
     * @param detail
     * @param inTime          计费开始时间
     * @param freeTime        免费时长（分钟）
     * @param maxOneTimeMoney 一次停车的最高费用（单位分）
     * @return
     */
    public Integer intervalPayableMoney(IntervalChargeDetail detail, Date inTime,
                                        Integer freeTime, Integer maxOneTimeMoney) {
        inTime = DateUtils.getNextMinute(inTime, freeTime);
        long sumMoney = 0;
        switch (detail.frameType) {
            //按照分钟收费
            case 0:
                long minute = DateUtils.subDate(inTime, new Date(), 2);
                sumMoney = (minute / detail.intervalTime) * detail.money;
                break;
            //按照小时收费
            case 1:
                long hour = DateUtils.subDate(inTime, new Date(), 3);
                sumMoney = (hour / detail.intervalTime) * detail.money;
                break;
            //按照固定收费
            default:
                sumMoney = (DateUtils.subDate(inTime, new Date(), 2) / detail.intervalTime) * detail.money;
                break;
        }
        sumMoney = sumMoney > maxOneTimeMoney ? maxOneTimeMoney : sumMoney;
        return (int) sumMoney;

    }

    /**
     * 按照间隔收费
     * "frameType": 0 按分钟收费
     * "frameType": 1 按小时收费
     */
    static class IntervalChargeDetail {
        private Integer money;
        private Integer frameType;
        //间隔时长
        private Integer intervalTime;

        public Integer getMoney() {
            return money;
        }

        public void setMoney(Integer money) {
            this.money = money;
        }

        public Integer getFrameType() {
            return frameType;
        }

        public void setFrameType(Integer frameType) {
            this.frameType = frameType;
        }

        public Integer getIntervalTime() {
            return intervalTime;
        }

        public void setIntervalTime(Integer intervalTime) {
            this.intervalTime = intervalTime;
        }
    }
}
