package com.huaching.parking.local.parking_local_general.domain.local.dto.request;

import com.huaching.parking.local.parking_local_general.domain.local.po.ThsParkingRecordsPO;

import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @program: parking_local_general
 * @description: 入场DTO
 * 本地入场DTO,所有的入场上报记录都要转换为此DTO RequestDTO
 * @author: lidongsheng
 * @createData: 2020-04-17 13:17
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-17 13:17
 * @updateContent: 入场上报DTO
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */

public class ThsCarInDTO {

    /**
     * 车场编号
     */
    @NotNull(message = "parkCode 车场编号 不能为空")
    private String parkCode;

    /**
     * 车牌号
     */
    @NotNull(message = "carNo 车牌号 不能为空")
    private String carNo;

    /**
     * 进场时间
     */
    @NotNull(message = "inTime 进场时间 不能为空")
    private Date inTime;

    /**
     * 进场的通道id
     */
    @NotNull(message = "inChannelCode 进场的通道编号 不能为空")
    private String inChannelCode;

    /**
     * 车辆类型：
     * <p>
     * 1 小型车
     * <p>
     * 2 大型车
     * <p>
     * 3 其他
     * <p>
     * 4 未识别
     */
    private Integer carType;

    /**
     * 车辆颜色：
     * <p>
     * 0蓝色；1黄色；2白色；3黑色；4绿色
     */
    private Integer carColor;

    /**
     * 入场车辆抓拍图片
     */
    private String inImgUrl;

    public String getParkCode() {
        return parkCode;
    }

    public void setParkCode(String parkCode) {
        this.parkCode = parkCode;
    }

    public String getCarNo() {
        return carNo;
    }

    public void setCarNo(String carNo) {
        this.carNo = carNo;
    }

    public Date getInTime() {
        return inTime;
    }

    public void setInTime(Date inTime) {
        this.inTime = inTime;
    }

    public String getInChannelCode() {
        return inChannelCode;
    }

    public void setInChannelCode(String inChannelCode) {
        this.inChannelCode = inChannelCode;
    }

    public Integer getCarType() {
        return carType;
    }

    public void setCarType(Integer carType) {
        this.carType = carType;
    }

    public Integer getCarColor() {
        return carColor;
    }

    public void setCarColor(Integer carColor) {
        this.carColor = carColor;
    }

    public String getInImgUrl() {
        return inImgUrl;
    }

    public void setInImgUrl(String inImgUrl) {
        this.inImgUrl = inImgUrl;
    }

    /**
     * PO转换为DTO
     *
     * @param thsParkingRecordsPO
     * @return
     */
    public static ThsCarInDTO poConvertToDto(ThsParkingRecordsPO thsParkingRecordsPO) {
        ThsCarInDTO thsCarInDTO = new ThsCarInDTO();
        thsCarInDTO.setCarNo(thsParkingRecordsPO.getCarNo());
        thsCarInDTO.setParkCode(thsParkingRecordsPO.getParkCode());
        thsCarInDTO.setCarColor(thsParkingRecordsPO.getCarColor());
        thsCarInDTO.setCarType(thsParkingRecordsPO.getCarType());
        thsCarInDTO.setInChannelCode(thsParkingRecordsPO.getInChannelCode());
        thsCarInDTO.setInTime(thsParkingRecordsPO.getInTime());
        thsCarInDTO.setInImgUrl(thsParkingRecordsPO.getInImgUrl());
        return thsCarInDTO;
    }

    /**
     * DTO存在的数据转换为PO
     *
     * @param thsCarInDTO
     * @return
     */
    public static ThsParkingRecordsPO dtoConvertToPo(ThsCarInDTO thsCarInDTO) {
        ThsParkingRecordsPO thsParkingRecordsPO = new ThsParkingRecordsPO();
        thsParkingRecordsPO.setCarNo(thsCarInDTO.getCarNo());
        thsParkingRecordsPO.setParkCode(thsCarInDTO.getParkCode());
        thsParkingRecordsPO.setCarColor(thsCarInDTO.getCarColor());
        thsParkingRecordsPO.setCarType(thsCarInDTO.getCarType());
        thsParkingRecordsPO.setInChannelCode(thsCarInDTO.getInChannelCode());
        thsParkingRecordsPO.setInTime(thsCarInDTO.getInTime());
        thsParkingRecordsPO.setInImgUrl(thsCarInDTO.getInImgUrl());
        thsParkingRecordsPO.setOnParkStatus(1);
        thsParkingRecordsPO.setCreateTime(new Date());
        thsParkingRecordsPO.setPayedMoney(0);
        thsParkingRecordsPO.setTotalMoney(0);
        thsParkingRecordsPO.setPayUpStatus(0);
        return thsParkingRecordsPO;
    }
}
