package com.huaching.parking.local.parking_local_general.domain.local.dto.request;

import com.huaching.parking.local.parking_local_general.domain.local.po.ThsParkingRecordsPO;

import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @program: parking_local_general
 * @description: 出场DTO
 * 本地出场DTO,所有的出场上报记录都要转换为此DTO RequestDTO
 * @author: lidongsheng
 * @createData: 2020-04-17 13:17
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-17 13:17
 * @updateContent: 入场上报DTO
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */

public class ThsCarOutDTO {

    /**
     * 车场编号
     */
    @NotNull(message = "车场编号 不能为空")
    private String parkCode;

    /**
     * 车牌号
     */
    @NotNull(message = "车牌号 不能为空")
    private String carNo;

    /**
     * 出场时间
     */
    @NotNull(message = "出场时间 不能为空")
    private Date outTime;

    /**
     * 出场的通道编号
     */
    @NotNull(message = "出场的通道编号 不能为空")
    private String outChannelCode;

    /**
     * 出场车辆抓拍图片
     */
    private String outImgUrl;

    public String getParkCode() {
        return parkCode;
    }

    public void setParkCode(String parkCode) {
        this.parkCode = parkCode;
    }

    public String getCarNo() {
        return carNo;
    }

    public void setCarNo(String carNo) {
        this.carNo = carNo;
    }

    public Date getOutTime() {
        return outTime;
    }

    public void setOutTime(Date outTime) {
        this.outTime = outTime;
    }

    public String getOutChannelCode() {
        return outChannelCode;
    }

    public void setOutChannelCode(String outChannelCode) {
        this.outChannelCode = outChannelCode;
    }

    public String getOutImgUrl() {
        return outImgUrl;
    }

    public void setOutImgUrl(String outImgUrl) {
        this.outImgUrl = outImgUrl;
    }

    /**
     * PO转换为DTO
     * @param thsParkingRecordsPO
     * @return
     */
    public static ThsCarOutDTO poConvertToDto(ThsParkingRecordsPO thsParkingRecordsPO){
        ThsCarOutDTO thsCarOutDTO = new ThsCarOutDTO();
        thsCarOutDTO.setCarNo(thsParkingRecordsPO.getCarNo());
        thsCarOutDTO.setParkCode(thsParkingRecordsPO.getParkCode());
        thsCarOutDTO.setOutTime(thsParkingRecordsPO.getOutTime());
        thsCarOutDTO.setOutChannelCode(thsParkingRecordsPO.getOutChannelCode());
        thsCarOutDTO.setOutImgUrl(thsParkingRecordsPO.getOutImgUrl());
        return thsCarOutDTO;
    }

    /**
     * DTO存在的数据转换为PO
     * @param thsCarOutDTO
     * @return
     */
    public static ThsParkingRecordsPO dtoConvertToPo(ThsCarOutDTO thsCarOutDTO,ThsParkingRecordsPO thsParkingRecordsPO){
        thsParkingRecordsPO.setOutTime(thsCarOutDTO.getOutTime());
        thsParkingRecordsPO.setOutChannelCode(thsCarOutDTO.getOutChannelCode());
        thsParkingRecordsPO.setOutImgUrl(thsCarOutDTO.getOutImgUrl());
        thsParkingRecordsPO.setOnParkStatus(0);
        thsParkingRecordsPO.setUpdateTime(new Date());
        return thsParkingRecordsPO;
    }

}
