package com.huaching.parking.local.parking_local_general.domain.local.dto.request;

import com.huaching.parking.transfer.domain.dto.PlatformPayChargeDto;
import com.huaching.parking.transfer.domain.dto.PlatformWhiteListSyncDto;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @program: parking_local_general
 * @description: 支付同步DTO
 * @author: lidongsheng
 * @createData: 2020-04-27 14:36
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-27 14:36
 * @updateContent: 支付同步DTO
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */

public class ThsPaySyncDTO {


    /**
     * 车场编号
     */
    @NotNull(message = "parkCode 车场编号 不能为空")
    private String parkCode;
    /**
     * 车牌号
     */
    @NotNull(message = "carNo 车牌号 不能为空")
    private String carNo;
    /**
     * 支付记录账单号
     */
    private String orderNo;
    /**
     * 支付金额(单位：分)
     */
    @NotNull(message = "payedMoney 支付金额 不能为空")
    private Integer payedMoney;

    /**
     * 优惠金额
     */
    private Integer couponMoney;

    public Integer getCouponMoney() {
        return couponMoney;
    }

    public void setCouponMoney(Integer couponMoney) {
        this.couponMoney = couponMoney;
    }

    /**
     * 应支付金额(单位：分)
     */
    private Integer payableMoney;
    /**
     * 支付时间
     */
    private Date payTime;

    /**
     * 支付类型：
     * <p>
     * 1：支付宝 2：微信 3：现金 4：其他
     */
    private Integer payType;

    /**
     * 支付终端类型:
     * <p>
     * 11：支付宝小程序 12：支付宝生活号
     * <p>
     * 21:  微信小程序 22：微信公众号
     * <p>
     * 31：现金
     */
    private Integer terminalType;

    /**
     * 操作人
     */
    private String operatorUser;

    /**
     * 备注字段
     */
    private String remake;

    public String getOperatorUser() {
        return operatorUser;
    }

    public void setOperatorUser(String operatorUser) {
        this.operatorUser = operatorUser;
    }

    public String getRemake() {
        return remake;
    }

    public void setRemake(String remake) {
        this.remake = remake;
    }

    public String getOrderNo() {
        return orderNo;
    }

    public void setOrderNo(String orderNo) {
        this.orderNo = orderNo;
    }

    public Integer getPayedMoney() {
        return payedMoney;
    }

    public void setPayedMoney(Integer payedMoney) {
        this.payedMoney = payedMoney;
    }

    public Integer getPayableMoney() {
        return payableMoney;
    }

    public void setPayableMoney(Integer payableMoney) {
        this.payableMoney = payableMoney;
    }

    public Date getPayTime() {
        return payTime;
    }

    public void setPayTime(Date payTime) {
        this.payTime = payTime;
    }

    public Integer getPayType() {
        return payType;
    }

    public void setPayType(Integer payType) {
        this.payType = payType;
    }

    public Integer getTerminalType() {
        return terminalType;
    }

    public void setTerminalType(Integer terminalType) {
        this.terminalType = terminalType;
    }

    public String getCarNo() {
        return carNo;
    }

    public void setCarNo(String carNo) {
        this.carNo = carNo;
    }

    public String getParkCode() {
        return parkCode;
    }

    public void setParkCode(String parkCode) {
        this.parkCode = parkCode;
    }

    public static ThsPaySyncDTO dtoConvertToThis(PlatformPayChargeDto dto) {
        ThsPaySyncDTO paySyncDTO = new ThsPaySyncDTO();
        paySyncDTO.setParkCode(dto.getPlatformParkId().toString());
        paySyncDTO.setCarNo(dto.getCarNo());
        paySyncDTO.setPayableMoney(new BigDecimal(dto.getReceivable() * 100).setScale(0, BigDecimal.ROUND_HALF_UP).intValue());
        paySyncDTO.setPayedMoney(new BigDecimal(dto.getReceived() * 100).setScale(0, BigDecimal.ROUND_HALF_UP).intValue());
        paySyncDTO.setPayTime(dto.getPayChargeTime());
        paySyncDTO.setPayType(dto.getPayType());
        paySyncDTO.setTerminalType(dto.getPayType());
        paySyncDTO.setOperatorUser("oneself");
        return paySyncDTO;
    }
}
