package com.huaching.parking.local.parking_local_general.domain.local.dto.response;

import com.alibaba.fastjson.JSON;
import com.huaching.parking.local.parking_local_general.domain.local.bo.ThsRequestChargeBO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.BaseDTO;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsChargingRulesPO;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsParkingRecordsPO;
import com.huaching.parking.local.parking_local_general.service.local.PaymentChargesServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;
import java.util.UUID;

/**
 * @program: parking_local_general
 * @description: 请求费用返回DTO
 * @author: lidongsheng
 * @createData: 2020-04-22 14:11
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-22 14:11
 * @updateContent: 请求费用信息DTO
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */

public class ThsRequestChargeDTO extends BaseDTO {

    private static final Logger logger = LoggerFactory.getLogger(ThsRequestChargeDTO.class);

    private String parkCode;

    private String carNo;

    /**
     * 进场时间
     */
    private Date inTime;

    public Date getOutTime() {
        return outTime;
    }

    public void setOutTime(Date outTime) {
        this.outTime = outTime;
    }

    /**
     * 出场时间（出口付的时候会用到）
     */
    private Date outTime;

    /**
     * 进场的通道编号
     */
    private String inChannelCode;

    /**
     * 车辆在场状态： 0 已出场  1 在场
     */
    private Integer onParkStatus;

    /**
     * 进场抓拍图片url
     */
    private String inImgUrl;

    /**
     * 账单号
     */
    private String orderNo;

    /**
     * 已支付金额
     */
    private Integer payedMoney;

    /**
     * 应支付金额
     */
    private Integer payableMoney;

    /**
     * 优惠金额
     */
    private Integer couponMoney;

    public String getErrMes() {
        return errMes;
    }

    public void setErrMes(String errMes) {
        this.errMes = errMes;
    }

    /**
     * 错误信息
     */
    private String errMes;

    public String getParkCode() {
        return parkCode;
    }

    public void setParkCode(String parkCode) {
        this.parkCode = parkCode;
    }

    public String getCarNo() {
        return carNo;
    }

    public void setCarNo(String carNo) {
        this.carNo = carNo;
    }

    public Date getInTime() {
        return inTime;
    }

    public void setInTime(Date inTime) {
        this.inTime = inTime;
    }

    public String getInChannelCode() {
        return inChannelCode;
    }

    public void setInChannelCode(String inChannelCode) {
        this.inChannelCode = inChannelCode;
    }

    public Integer getOnParkStatus() {
        return onParkStatus;
    }

    public void setOnParkStatus(Integer onParkStatus) {
        this.onParkStatus = onParkStatus;
    }

    public String getInImgUrl() {
        return inImgUrl;
    }

    public void setInImgUrl(String inImgUrl) {
        this.inImgUrl = inImgUrl;
    }

    public String getOrderNo() {
        return orderNo;
    }

    public void setOrderNo(String orderNo) {
        this.orderNo = orderNo;
    }

    public Integer getPayedMoney() {
        return payedMoney;
    }

    public void setPayedMoney(Integer payedMoney) {
        this.payedMoney = payedMoney;
    }

    public Integer getPayableMoney() {
        return payableMoney;
    }

    public void setPayableMoney(Integer payableMoney) {
        this.payableMoney = payableMoney;
    }

    public Integer getCouponMoney() {
        return couponMoney;
    }

    public void setCouponMoney(Integer couponMoney) {
        this.couponMoney = couponMoney;
    }

    public static ThsRequestChargeDTO poConvertToDto(ThsParkingRecordsPO thsParkingRecordsPO, String orderNo, Integer payableMoney) {
        logger.info("poConvertToDto 开始转换 po -> {} ,orderNo -> {}, payableMoney -> {}",
                JSON.toJSONString(thsParkingRecordsPO), orderNo, payableMoney);
        ThsRequestChargeDTO dto = new ThsRequestChargeDTO();
        String parkCode = thsParkingRecordsPO.getParkCode();
        String carNo = thsParkingRecordsPO.getCarNo();
        dto.setParkCode(parkCode);
        dto.setCarNo(carNo);
        dto.setInChannelCode(thsParkingRecordsPO.getInChannelCode());
        dto.setInTime(thsParkingRecordsPO.getInTime());
        dto.setPayedMoney(thsParkingRecordsPO.getPayedMoney());
        dto.setCouponMoney(thsParkingRecordsPO.getCouponMoney());
        dto.setInImgUrl(thsParkingRecordsPO.getInImgUrl());
        dto.setOnParkStatus(thsParkingRecordsPO.getOnParkStatus());
        dto.setOrderNo(orderNo);
        dto.setPayableMoney(payableMoney);
        logger.info("poConvertToDto 转换完成  dto -> {}",
                JSON.toJSONString(dto));
        return dto;
    }

    /**
     * 返回错误
     *
     * @param errorMsg 错误信息
     * @return
     */
    public static ThsRequestChargeDTO errorMsg(String errorMsg) {
        ThsRequestChargeDTO requestChargeDTO = new ThsRequestChargeDTO();
        requestChargeDTO.setCode(0);
        requestChargeDTO.setMsg(errorMsg);
        return requestChargeDTO;
    }
}
