package com.huaching.parking.local.parking_local_general.domain.zhaf.dto;

import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.serializer.PropertyFilter;
import com.huaching.parking.local.parking_local_general.config.zhaf.DZConfig;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsCarInDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsCarOutDTO;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsCarAdvanceInOutRecordsPO;
import com.huaching.parking.local.parking_local_general.factory.zhaf.ThreadPoolFactory;
import com.huaching.parking.local.parking_local_general.util.local.DateUtils;
import com.huaching.parking.local.parking_local_general.util.zhaf.Base64Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.util.ResourceUtils;

import java.io.FileNotFoundException;
import java.util.Date;

/**
 * @program: parking_local_general
 * @description: 道闸过车上报DTO
 * @author: lidongsheng
 * @createData: 2020-04-14 16:02
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-14 16:02
 * @updateContent: 道闸过车上报DTO
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */

public class BarriergateReportDTO {


    private static final String SAVE_IMG_ROOT_URL = "barriergateReport/";

    private static final Logger logger = LoggerFactory.getLogger(BarriergateReportDTO.class);
    /**
     * swipeTime : 0
     * channelNo : 0
     * direction : 0
     * confidence : 0
     * useTime : 0
     * deviceInfo : {"deviceCode":"string","serialNumber":"string","deviceVersion":"string","deviceName":"string","ip":"string"}
     * carInfo : {"plateNumber":"string","platerColor":"string","platerType":"string","carColor":"string","plateImg":"string","carImg":"string"}
     */

    private String swipeTime;
    private String channelNo;
    private int direction;
    private int confidence;
    private String useTime;
    private DeviceInfoBean deviceInfo;
    private CarInfoBean carInfo;

    public String getSwipeTime() {
        return swipeTime;
    }

    public void setSwipeTime(String swipeTime) {
        this.swipeTime = swipeTime;
    }

    public String getChannelNo() {
        return channelNo;
    }

    public void setChannelNo(String channelNo) {
        this.channelNo = channelNo;
    }

    public String getUseTime() {
        return useTime;
    }

    public void setUseTime(String useTime) {
        this.useTime = useTime;
    }

    public int getDirection() {
        return direction;
    }

    public void setDirection(int direction) {
        this.direction = direction;
    }

    public int getConfidence() {
        return confidence;
    }

    public void setConfidence(int confidence) {
        this.confidence = confidence;
    }

    public DeviceInfoBean getDeviceInfo() {
        return deviceInfo;
    }

    public void setDeviceInfo(DeviceInfoBean deviceInfo) {
        this.deviceInfo = deviceInfo;
    }

    public CarInfoBean getCarInfo() {
        return carInfo;
    }

    public void setCarInfo(CarInfoBean carInfo) {
        this.carInfo = carInfo;
    }

    public static class DeviceInfoBean {
        /**
         * deviceCode : string
         * serialNumber : string
         * deviceVersion : string
         * deviceName : string
         * ip : string
         */

        private String deviceCode;
        private String serialNumber;
        private String deviceVersion;
        private String deviceName;
        private String ip;

        public String getDeviceCode() {
            return deviceCode;
        }

        public void setDeviceCode(String deviceCode) {
            this.deviceCode = deviceCode;
        }

        public String getSerialNumber() {
            return serialNumber;
        }

        public void setSerialNumber(String serialNumber) {
            this.serialNumber = serialNumber;
        }

        public String getDeviceVersion() {
            return deviceVersion;
        }

        public void setDeviceVersion(String deviceVersion) {
            this.deviceVersion = deviceVersion;
        }

        public String getDeviceName() {
            return deviceName;
        }

        public void setDeviceName(String deviceName) {
            this.deviceName = deviceName;
        }

        public String getIp() {
            return ip;
        }

        public void setIp(String ip) {
            this.ip = ip;
        }
    }

    public static class CarInfoBean {
        /**
         * plateNumber : string
         * platerColor : string
         * platerType : string
         * carColor : string
         * plateImg : string
         * carImg : string
         */

        private String plateNumber;
        private String platerColor;
        private String platerType;
        private String carColor;
        @JSONField(serialize = false)
        private String plateImg;
        @JSONField(serialize = false)
        private String carImg;
        private String byListType;

        public String getPlateNumber() {
            return plateNumber;
        }

        public void setPlateNumber(String plateNumber) {
            this.plateNumber = plateNumber;
        }

        public String getPlaterColor() {
            return platerColor;
        }

        public void setPlaterColor(String platerColor) {
            this.platerColor = platerColor;
        }

        public String getPlaterType() {
            return platerType;
        }

        public void setPlaterType(String platerType) {
            this.platerType = platerType;
        }

        public String getCarColor() {
            return carColor;
        }

        public void setCarColor(String carColor) {
            this.carColor = carColor;
        }

        public String getPlateImg() {
            return plateImg;
        }

        public void setPlateImg(String plateImg) {
            this.plateImg = plateImg;
        }

        public String getCarImg() {
            return carImg;
        }

        public void setCarImg(String carImg) {
            this.carImg = carImg;
        }

        public String getByListType() {
            return byListType;
        }

        public void setByListType(String byListType) {
            this.byListType = byListType;
        }
    }

    /**
     * 转换为ThsCarInDTO
     *
     * @param barriergateReportDTO
     * @return
     */
    public static ThsCarInDTO convertThsCarInDTO(BarriergateReportDTO barriergateReportDTO) {
        ThsCarInDTO thsCarInDTO = new ThsCarInDTO();
        thsCarInDTO.setCarNo(barriergateReportDTO.getCarInfo().getPlateNumber());
        thsCarInDTO.setParkCode(DZConfig.getParkCode());
//        thsCarInDTO.setCarColor(barriergateReportDTO.getCarInfo().carColor);
//        thsCarInDTO.setCarType(barriergateReportDTO.getCarInfo());
        thsCarInDTO.setInChannelCode(String.valueOf(barriergateReportDTO.deviceInfo.getDeviceCode()));
        thsCarInDTO.setInTime(new Date());
        return thsCarInDTO;
    }

    /**
     * 转换为ThsCarAdvanceInOutRecordsPO(预出入场)
     *
     * @param barriergateReportDTO
     * @return
     */
    public static ThsCarAdvanceInOutRecordsPO advanceThsCarInDTO(BarriergateReportDTO barriergateReportDTO, int direction) {
        ThsCarAdvanceInOutRecordsPO thsCarAdvanceInOutRecordsPO = new ThsCarAdvanceInOutRecordsPO();
        if (direction == 3) {
            if (barriergateReportDTO.getDirection() == 3) {
                thsCarAdvanceInOutRecordsPO.setCapFlag(1);
            } else {
                thsCarAdvanceInOutRecordsPO.setCapFlag(2);
            }
        } else {
            thsCarAdvanceInOutRecordsPO.setCapFlag(direction);
        }
        thsCarAdvanceInOutRecordsPO.setCapChannelCode(String.valueOf(barriergateReportDTO.deviceInfo.getDeviceCode()));
        thsCarAdvanceInOutRecordsPO.setCapTime(new Date());

        /**** 图片处理 开始 ****/
        String nowDate = DateUtils.dateFormatToString(new Date(), 3);
        String saveUrl = DZConfig.getSaveImgUrl() + SAVE_IMG_ROOT_URL + nowDate;
        String imgName = barriergateReportDTO.getDeviceInfo().getDeviceCode() + "_"
                + barriergateReportDTO.getCarInfo().getPlateNumber() + "_" + barriergateReportDTO.getSwipeTime() + ".jpg";
        if (barriergateReportDTO.getCarInfo().getCarImg() != null) {
            thsCarAdvanceInOutRecordsPO.setCapImgUrl("/img/nowDate/" + imgName);
        }
        ThreadPoolFactory.getGeneral().execute(() -> {
            Base64Utils.GenerateImage(barriergateReportDTO.getCarInfo().getCarImg(),
                    saveUrl, imgName);
        });
        /**** 图片处理 结束 ****/

        thsCarAdvanceInOutRecordsPO.setCarNo(barriergateReportDTO.getCarInfo().getPlateNumber());
        thsCarAdvanceInOutRecordsPO.setParkCode(DZConfig.getParkCode());
        return thsCarAdvanceInOutRecordsPO;
    }

    /**
     * PO转换为DTO
     *
     * @param barriergateReportDTO
     * @return
     */
    public static ThsCarOutDTO convertThsCarOutDTO(BarriergateReportDTO barriergateReportDTO) {
        ThsCarOutDTO thsCarOutDTO = new ThsCarOutDTO();
        thsCarOutDTO.setCarNo(barriergateReportDTO.getCarInfo().getPlateNumber());
        thsCarOutDTO.setParkCode(DZConfig.getParkCode());
        thsCarOutDTO.setOutChannelCode(String.valueOf(barriergateReportDTO.deviceInfo.getDeviceCode()));
        thsCarOutDTO.setOutTime(new Date());
        return thsCarOutDTO;
    }
}
