package com.huaching.parking.local.parking_local_general.enums.local;

import java.util.ArrayList;
import java.util.List;

/**
 * @Program:
 * @Description:
 * @Author: lidongsheng
 * @CreateData: 2019年10月8日13:12:17
 * @UpdateAuthor:
 * @UpdateData:
 * @UpdateContent:
 * @Version: 1.0
 */
public enum ReasonCodeEnums {
    /**
     * 调用接口参数错误
     */
    REQUEST_PARM_VALID_DEFEAT(40001, "接口请求参数校验未通过"),

    /**
     * 处理逻辑错误
     */
    EMPTY_RESULT(41001, "根据传入条件未查询到结果"),

    /**
     * 没有在场记录
     */
    NO_CAR_IN(41002, "没有在场记录"),

    /**
     * 没有配置收费规则
     */
    NO_PARK_CHARGE_CONFIG(41003, "没有配置收费规则"),

    /**
     * 通道信息不存在
     */
    NO_PARK_CHANNEL_CONFIG(41004, "通道信息不存在"),

    /**
     * 通道信息不存在
     */
    NO_ADVANCE_OUT_RECORDS(41005, "待出场记录不存在"),


    //支付相关 42000开头

    /**
     * 没有待支付订单记录
     */
    NO_UNPAID_PAY_ORDER_RECODE(42001, "没有待支付订单记录"),

    /**
     * oderNo已经过期
     */
    ORDER_NO_IS_EXPIRE(42002, "oderNo已经过期"),

    CHARGES_NOT_PAID(42002, "费用未缴清"),

    DB_OPERATION_ERROR(44441, "数据库操作异常"),


    CONTROL_BARRIER_GATE(44401, "控制道闸失败"),

    UNKNOWN(44444, "未知原因"),

    ;


    /**
     * 类型code
     */
    private final Integer code;

    /**
     * 类型说明
     */
    private final String message;

    /**
     * 构建一个 SchedulerTypeEnums
     *
     * @param code    类型code
     * @param message 类型说明
     */
    private ReasonCodeEnums(Integer code, String message) {
        this.code = code;
        this.message = message;
    }

    /**
     * 得到类型code
     *
     * @return 类型code
     */
    public Integer getCode() {
        return code;
    }

    /**
     * 得到类型说明。
     *
     * @return 类型说明。
     */
    public String getMessage() {
        return message;
    }

    /**
     * 通过枚举值码查找枚举值。
     *
     * @param code 查找枚举值的枚举值码。
     * @return 枚举值码对应的枚举值。
     */
    public static ReasonCodeEnums findPushType(Integer code) {
        for (ReasonCodeEnums pushType : values()) {
            if (pushType.getCode() == code) {
                return pushType;
            }
        }
        // 如果找不到枚举值，那就返回null
        return null;
    }

    /**
     * 获取全部枚举值。
     *
     * @return 全部枚举值。
     */
    public static List<ReasonCodeEnums> getAllPushType() {
        List<ReasonCodeEnums> list = new ArrayList<>();
        for (ReasonCodeEnums pushType : values()) {
            list.add(pushType);
        }
        return list;
    }

    /**
     * 获取全部枚举值码。
     *
     * @return 全部枚举值码。
     */
    public static List<Integer> getAllPushTypeCode() {
        List<Integer> list = new ArrayList<>();
        for (ReasonCodeEnums pushType : values()) {
            list.add(pushType.getCode());
        }
        return list;
    }

    /**
     * 重写toString，返回code
     *
     * @return code
     */
    @Override
    public String toString() {
        return code.toString();
    }
}
