package com.huaching.parking.local.parking_local_general.enums.local;

import java.util.ArrayList;
import java.util.List;

/**
 * @program: parking_local_general
 * @description: 收费规则enum
 * @author: lidongsheng
 * @createData: 2020-05-19 17:16
 * @updateAuthor: lidongsheng
 * @updateData: 2020-05-19 17:16
 * @updateContent: 收费规则类型enum
 * @Version: 1.0.0
 * @email: lidongshenglife@163.com
 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */

public enum RequestChargeTypeEnums {

    /**
     * 调用接口参数错误
     */
    ONE_TIME(0, "按次收费"),

    /**
     * 处理逻辑错误
     */
    TIME_FRAME(1, "按照时段收费"),

    /**
     * 没有在场记录
     */
    INTERVAL(2, "按照时间间隔计算费用");


    /**
     * 类型code
     */
    private final Integer code;

    /**
     * 类型说明
     */
    private final String message;

    /**
     * 构建一个 SchedulerTypeEnums
     *
     * @param code    类型code
     * @param message 类型说明
     */
    private RequestChargeTypeEnums(Integer code, String message) {
        this.code = code;
        this.message = message;
    }

    /**
     * 得到类型code
     *
     * @return 类型code
     */
    public Integer getCode() {
        return code;
    }

    /**
     * 得到类型说明。
     *
     * @return 类型说明。
     */
    public String getMessage() {
        return message;
    }

    /**
     * 通过枚举值码查找枚举值。
     *
     * @param code 查找枚举值的枚举值码。
     * @return 枚举值码对应的枚举值。
     */
    public static RequestChargeTypeEnums findPushType(Integer code) {
        for (RequestChargeTypeEnums pushType : values()) {
            if (pushType.getCode().equals(code)) {
                return pushType;
            }
        }
        // 如果找不到枚举值，那就返回null
        return null;
    }

    /**
     * 获取全部枚举值。
     *
     * @return 全部枚举值。
     */
    public static List<RequestChargeTypeEnums> getAllPushType() {
        List<RequestChargeTypeEnums> list = new ArrayList<>();
        for (RequestChargeTypeEnums RequestChargeType : values()) {
            list.add(RequestChargeType);
        }
        return list;
    }

    /**
     * 获取全部枚举值码。
     *
     * @return 全部枚举值码。
     */
    public static List<Integer> getAllCode() {
        List<Integer> list = new ArrayList<>();
        for (RequestChargeTypeEnums RequestChargeType : values()) {
            list.add(RequestChargeType.getCode());
        }
        return list;
    }

}
