package com.huaching.parking.local.parking_local_general.longConnection.client;

import com.huaching.parking.local.parking_local_general.longConnection.util.SocketManager;
import com.huaching.parking.transfer.client.IClient;
import com.huaching.parking.transfer.common.EventCategory;
import com.huaching.parking.transfer.common.EventType;
import com.huaching.parking.transfer.protocol.Request;
import com.huaching.parking.transfer.util.SynchUtil;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioSocketChannel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * Created by xiajl on        .
 */
public class NettyClient implements IClient {

    private Logger logger = LoggerFactory.getLogger(NettyClient.class);
    private String host;
    private int port;
    private Channel channel = null;
    private EventLoopGroup workerGroup;
    private Bootstrap b;
    private String clientName;
    private ChannelInitializer channelInitializer;
    private boolean send;
    private Request initReq = new Request();

    public NettyClient(String host, int port, String clientName, ChannelInitializer channelInitializer, boolean send) {
        this.host = host;
        this.port = port;
        this.send = send;
        this.channelInitializer = channelInitializer;
        this.clientName = clientName;
    }


    @Override
    public void connect(){
        try{
            logger.debug("connect！！");
            if (channel != null && channel.isActive()) {
                logger.info("已经存在连接。。");
                return;
            }

            ChannelFuture future = b.connect(host, port);


            System.out.println(System.getProperty("file.encoding"));
            logger.info(clientName + "tcpClient start  初始地址为：" + host +" : " +port);
            future.addListener((ChannelFutureListener) futureListener -> {
                if (futureListener.isSuccess()) {
                    channel = futureListener.channel();
                    initMsgAndSend();
                    SocketManager.getInstance().addClient(clientName,channel);
                    logger.info(clientName +" 连接成功！");
                } else {
                    logger.info(clientName+" 连接失败, 15s后尝试重连");
                    futureListener.channel().eventLoop().schedule(this::connect, 15, TimeUnit.SECONDS);
                }
            });

        }catch (Exception e){
            logger.error(e.getMessage(),e);
            SocketManager.getInstance().removeClient(clientName);
        }

    }

    private void initMsgAndSend() {
        if(!send){
            return;
        }
        initReq.setType(EventType.ASK);
        initReq.setCategory(EventCategory.COMMAND);
//        JSONObject obj = new JSONObject();
//        obj.put(SynchUtil.REQUEST_TYPE_FILED, "i");
//        obj.put(SynchUtil.REQUEST_DATA_FILED,getName());
        initReq.setRequestId(SynchUtil.getUUIDRequestId());
        initReq.setData(getName());
        initReq.setDataType("i");
        new Thread(() -> sendInitMsg(initReq)).start();
    }

    private void sendInitMsg(final Request initReq) {
        try {
            if(channel == null || !channel.isActive()){
                //通道失效，退出
                return;
            }
            logger.info(getName() + " 发送初始消息");
            String result = Optional.ofNullable(SynchUtil.syncRequest(channel, initReq)).filter(response -> response.getCode() == 1)
                    .orElseThrow(()->new Exception(getName() +" 初始消息发送失败！")).getResult();
            logger.info(getName() + "  " + result);
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            try {
                TimeUnit.SECONDS.sleep(5L);
            } catch (InterruptedException e1) {
            }
            sendInitMsg(initReq);
        }
    }

    @Override
    public void start() {
        workerGroup = new NioEventLoopGroup();
        try {
            b = new Bootstrap(); //
        b.group(workerGroup); //
            b.channel(NioSocketChannel.class); //
            b.option(ChannelOption.SO_KEEPALIVE, true); //
            b.handler(channelInitializer);
            // 启动客户端
           connect();
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            if(workerGroup != null){
                workerGroup.shutdownGracefully();
            }
            SocketManager.getInstance().removeClient(clientName);
        }
    }


    @Override
    public void close(){
        if(workerGroup != null){
            workerGroup.shutdownGracefully();
        }
        SocketManager.getInstance().removeClient(clientName);
    }

    @Override
    public String getName() {
        return clientName;
    }


}
