package com.huaching.parking.local.parking_local_general.longConnection.handler;

import com.huaching.parking.local.parking_local_general.longConnection.listener.MessageListener;
import com.huaching.parking.local.parking_local_general.longConnection.util.SocketManager;
import com.huaching.parking.transfer.client.IClient;
import com.huaching.parking.transfer.listener.RequestListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.task.TaskExecutor;

import java.util.concurrent.TimeUnit;

/**
 * Created by xiajl on 2017/8/15.
 */
public abstract class AbstractLocalHandler extends ChannelInboundHandlerAdapter {

  private Logger logger = LoggerFactory.getLogger(AbstractLocalHandler.class);
  protected IClient client;
  protected RequestListener listener;
  protected MessageListener messageListener;

  private TaskExecutor threadPool;

  //protected Executor threadPool = Executors.newCachedThreadPool();


  @Override
  public void channelActive(ChannelHandlerContext ctx) throws Exception {
    SocketManager.getInstance().addClient(client.getName(), ctx.channel());
    super.channelActive(ctx);
  }

  @Override
  public void channelInactive(ChannelHandlerContext ctx) throws Exception {
    logger.info(client.getName() + "  与服务器断开连接");
    super.channelInactive(ctx);
    SocketManager.getInstance().removeClient(client.getName());
    //重新连接服务器
    ctx.channel().eventLoop().schedule(() -> {
      logger.info("尝试重新连接。。");
      client.connect();
    }, 3, TimeUnit.SECONDS);
       /* threadPool.execute(()->{
            logger.info("尝试重新连接。。");
            client.connect();
        });*/
    ctx.close();
  }

  @Override
  public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
    //心跳配置
    if (IdleStateEvent.class.isAssignableFrom(evt.getClass())) {
      IdleStateEvent event = (IdleStateEvent) evt;
      if (event.state() == IdleState.READER_IDLE) {
        //表示已经多久没有收到数据了
        //ctx.close();
      } else if (event.state() == IdleState.WRITER_IDLE) {
        //表示已经多久没有发送数据了
        sendHeartData(ctx);
      } else if (event.state() == IdleState.ALL_IDLE) {
        //表示已经多久既没有收到也没有发送数据了

      }
    }
  }

  protected abstract void sendHeartData(ChannelHandlerContext ctx);

  public void setClient(IClient client) {
    this.client = client;
  }

  public void addListener(RequestListener listener) {
    this.listener = listener;
  }

  public void addLocalListener(MessageListener listener) {
    this.messageListener = listener;
  }


  public TaskExecutor getThreadPool() {
    return threadPool;
  }

  public void setThreadPool(TaskExecutor threadPool) {
    this.threadPool = threadPool;
  }
}
