package com.huaching.parking.local.parking_local_general.longConnection.handler;

import com.huaching.parking.transfer.common.EventCategory;
import com.huaching.parking.transfer.common.EventType;
import com.huaching.parking.transfer.exception.RequestHandlerException;
import com.huaching.parking.transfer.protocol.Request;
import com.huaching.parking.transfer.protocol.Response;
import com.huaching.parking.transfer.util.SynchUtil;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

/**
 *
 * 本地与云平台的通道
 *
 * @author xiajl
 * @date 2017/7/28
 */
@ChannelHandler.Sharable
public class CloudConnectHandler extends AbstractLocalHandler {

    private Logger logger = LoggerFactory.getLogger(CloudConnectHandler.class);

    private Request heart = new Request();
    public CloudConnectHandler(){
        //心跳包

        heart.setType(EventType.NOTIFY);
        heart.setCategory(EventCategory.COMMAND);
        heart.setRequestId(SynchUtil.getUUIDRequestId());
//        JSONObject hjson = new JSONObject();
//        hjson.put(SynchUtil.REQUEST_TYPE_FILED, "h");
//        heart.setData(hjson.toJSONString());
        heart.setDataType("h");
        heart.setData("");
    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg)
            throws Exception {
        if (Response.class.isAssignableFrom(msg.getClass())) {
            Response response = (Response) msg;

            SynchUtil.setFutureResponse(response);
        } else if (Request.class.isAssignableFrom(msg.getClass())) {
            getThreadPool().execute(()->requestService((Request) msg, ctx));
        }

    }

    private void requestService(Request request, ChannelHandlerContext ctx) {
        if (request == null || StringUtils.isEmpty(request.getData())) {
            return;
        }
        Response response = null;
        String msg = "";
        try {
            request.setChannelId(ctx.channel().id().asLongText());
            response = listener.onRequest(request);
        }catch (RequestHandlerException h){
            logger.error(h.getMessage(), h);
            msg = h.getMessage();
        }catch (Exception e) {
            logger.error(e.getMessage(), e);
            msg = e.getMessage();
        }

        if (request.getType() == EventType.ASK || request.getCategory() == EventCategory.PUBLISH
            || request.getCategory() == EventCategory.PUBLISH_IMMEDIATE) {
            if(response == null){
                response = new Response();
                response.setCode((byte) 0);
                response.setMessage(msg);
            }
            response.setRequestId(request.getRequestId());
            SynchUtil.onResponse(ctx.channel(), response);
        }
    }
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause)
            throws Exception {
        logger.error(cause.getMessage(),cause);
    }

    @Override
    protected void sendHeartData(ChannelHandlerContext ctx) {
        try {
            SynchUtil.onRequest(ctx.channel(),heart);
//            logger.info("心跳数据---");
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        }
    }

}
