package com.huaching.parking.local.parking_local_general.longConnection.listener.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huaching.parking.local.parking_local_general.domain.ResponseResult;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsMemberCarDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsPaySyncDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.response.ThsRequestChargeDTO;
import com.huaching.parking.local.parking_local_general.longConnection.listener.CloudMessageListener;
import com.huaching.parking.local.parking_local_general.longConnection.util.SocketManager;
import com.huaching.parking.local.parking_local_general.service.local.MemberCarHelperService;
import com.huaching.parking.local.parking_local_general.service.local.PaymentChargesService;
import com.huaching.parking.transfer.common.EventCategory;
import com.huaching.parking.transfer.domain.dto.*;
import com.huaching.parking.transfer.domain.pojo.EventProcess;
import com.huaching.parking.transfer.enums.PlatformDataInterfaceTypeEnums;
import com.huaching.parking.transfer.protocol.Request;
import com.huaching.parking.transfer.protocol.Response;
import com.huaching.parking.transfer.service.EventProcessService;
import com.huaching.parking.transfer.util.SynchUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;

/**
 * 当有云平台数据下发时，触发 Created by xiajl on 2017/7/28.
 */
@Service
public class CloudConnectMessageProcess implements CloudMessageListener {

    private Logger logger = LoggerFactory.getLogger(CloudConnectMessageProcess.class);

    @Autowired
    private EventProcessService processService;

    @Autowired
    private PaymentChargesService paymentChargesService;

    @Autowired
    private MemberCarHelperService memberCarHelperService;


    @Override
    public Response onRequest(Request request) throws Exception {
        //接收请求，持久化数据，根据定时任务进行业务逻辑处理
        if (request.getCategory() == EventCategory.PUBLISH) {
            return handlerPublishEvent(request, (byte) 0);
            //接收请求，持久化数据，立即执行一次业务逻辑，后续根据定时任务进行业务逻辑处理
        } else if (request.getCategory() == EventCategory.PUBLISH_IMMEDIATE) {
            Response response = null;
            try {
                //执行命令
                response = doCommand(request);
            } catch (Exception e) {
                logger.error(e.getMessage(), e);
                response.setCode((byte) 0);
                response.setMessage("local端publishEvent持久化失败！");
            }

            return response;
        } else {
            //执行命令，数据不进行持久化操作
            return doCommand(request);
        }
    }

    private Response doCommand(Request request) throws Exception {

        String type = request.getDataType();

        if (type.equals(PlatformDataInterfaceTypeEnums.REQUEST_CHARGE.getType())) {
            logger.info("请求费用数据：" + request.getData());
            PlatformRequestChargeDto dto = JSON.parseObject(request.getData(), PlatformRequestChargeDto.class);
            ThsRequestChargeDTO thsRequestChargeDTO =
                    paymentChargesService.requestCharge(dto.getPlatformParkId().toString(), dto.getCarNo(), null);
            Response response = new Response();
            if (thsRequestChargeDTO.getCode() == 1) {
                response.setCode((byte) 1);
                response.setResult(String.valueOf(thsRequestChargeDTO.getPayableMoney()));
            } else {
                response.setCode((byte) 0);
                response.setMessage("请求费用失败");
            }
            return response;
        }

        //科拓的支付费用未用到 实际为订单同步
        else if (type.equals(PlatformDataInterfaceTypeEnums.PAY_CHARGE.getType())) {
            logger.info("支付同步下发：" + request.getData());
            PlatformPayChargeDto dto = JSON.parseObject(request.getData(), PlatformPayChargeDto.class);
            Response response = new Response();
            ResponseResult responseResult = paymentChargesService.paySync(ThsPaySyncDTO.dtoConvertToThis(dto));
            if (responseResult.getCompleteCode() == 1) {
                response.setCode((byte) 1);
                response.setMessage("支付同步成功");
            } else {
                response.setCode((byte) 0);
                response.setReasonCode(responseResult.getReasonCode().byteValue());
                response.setMessage(responseResult.getReasonMessage());
            }
            return response;

        } else if (type.equals(PlatformDataInterfaceTypeEnums.OFF_LINE_PAY_REQUEST_CHARGE.getType())) {
            logger.info("doCommand:执行二维码线下请求费用！");
            PlatformOfflinePayRequestChargeDto dto = JSON.parseObject(request.getData(), PlatformOfflinePayRequestChargeDto.class);
            ThsRequestChargeDTO thsRequestChargeDTO = paymentChargesService.outRequestCharge(dto.getPlatformParkId().toString(), dto.getChannelName());
            Response response = new Response();
            if (thsRequestChargeDTO.getCode() == 1) {
                response.setCode((byte) 1);
                JSONObject json = new JSONObject();
                json.put("MustFee", thsRequestChargeDTO.getPayableMoney());
                json.put("carNo", thsRequestChargeDTO.getCarNo());
                json.put("inTime", thsRequestChargeDTO.getInTime());
                json.put("outTime", thsRequestChargeDTO.getOutTime());
                json.put("hqParkId", thsRequestChargeDTO.getParkCode());
                response.setResult(json.toJSONString());
            } else {
                response.setCode((byte) 0);
                response.setMessage(thsRequestChargeDTO.getMsg());
            }
            return response;
        } else if (type.equals(PlatformDataInterfaceTypeEnums.COUPON_SEND_DOWN.getType())) {
            logger.info("优惠券下发");
            PlatformCouponSendDownDto dto = JSON.parseObject(request.getData(), PlatformCouponSendDownDto.class);
            Response response = new Response();
            response.setCode((byte) 0);
            response.setMessage("优惠券下发失败");
            return response;
        } else if (type.equals(PlatformDataInterfaceTypeEnums.OPEN_GATE_NOTICE.getType())) {
            PlatformOpenGateDto dto = JSON.parseObject(request.getData(), PlatformOpenGateDto.class);
            logger.info("远程开闸：" + JSON.toJSONString(dto));
            Response response = new Response();
            response.setCode((byte) 0);
            response.setMessage("远程开闸失败");
            return response;
        } else if (type.equals(PlatformDataInterfaceTypeEnums.WHITE_LIST_SYNC.getType())) {
            logger.info("会员卡权益下发：");
            PlatformWhiteListSyncDto dto = JSON.parseObject(request.getData(), PlatformWhiteListSyncDto.class);
            Response response = new Response();
            if (memberCarHelperService.memberSync(ThsMemberCarDTO.dtoConvertToThis(dto))) {
                response.setCode((byte) 1);
                response.setMessage("会员卡权益下发成功");
            } else {
                response.setCode((byte) 0);
                response.setMessage("会员卡权益下发失败");
            }
            return response;
        } else if (type.equals(PlatformDataInterfaceTypeEnums.WHITE_LIST_DELETE.getType())) {
            logger.info("会员卡权益删除：");
            PlatformWhiteListDeleteDto dto = JSON.parseObject(request.getData(), PlatformWhiteListDeleteDto.class);
            Response response = new Response();
            if (memberCarHelperService.memberDel(dto.getPlatformParkId().toString(), dto.getCarNo())) {
                response.setCode((byte) 1);
                response.setMessage("会员卡权益删除成功");
            } else {
                response.setCode((byte) 0);
                response.setMessage("会员卡权益删除失败");
            }
            return response;
        }

//        else if (type.equals(PlatformDataInterfaceTypeEnums.RESERVE.getType())) {
//            logger.debug("预约：" + request.getData());
//            ReserveSpaceDto dto = JSON.parseObject(request.getData(), ReserveSpaceDto.class);
//            return reserveSpace(dto);
//
//        } else if (type.equals(PlatformDataInterfaceTypeEnums.CANCEL_RESERVE.getType())) {
//            logger.debug("取消预约：" + JSON.toJSONString(request.getData()));
//            return cancelReserve(request.getData());
//
//        }
        else {
            Response response = new Response();
            response.setCode((byte) 0);
            response.setMessage("未知的接口类型！");
            return response;
        }

//        Response response = new Response();
//        response.setCode((byte)1);
//        return response;


    }

//    /**
//     * 预约处理
//     *
//     * @param dto
//     * @return
//     */
//    private Response reserveSpace(ReserveSpaceDto dto) {
//        return remoteDataService.reserveSpace(dto);
//    }
//
//    /**
//     * 取消预约
//     *
//     * @param carNo
//     * @return
//     */
//    private Response cancelReserve(String carNo) {
//        return remoteDataService.cancelReserve(carNo);
//    }

    /**
     * 将发布数据保存为待处理处理
     */
    private Response handlerPublishEvent(Request request, byte status) {
        Response response = new Response();
        EventProcess process = SynchUtil.coventFromRequest(request);
        process.setEventStatus(status);
        process.setChannelName(SocketManager.CLOUD_TCP_CLIENT);
        if (processService.insertEventProcess(process) > 0) {
            response.setCode((byte) 1);
        }
        return response;
    }


}
