package com.huaching.parking.local.parking_local_general.longConnection.runner;

import com.huaching.parking.local.parking_local_general.longConnection.client.NettyClient;
import com.huaching.parking.local.parking_local_general.longConnection.handler.AbstractLocalHandler;
import com.huaching.parking.local.parking_local_general.longConnection.handler.CloudConnectHandler;
import com.huaching.parking.local.parking_local_general.longConnection.listener.impl.CloudConnectMessageProcess;
import com.huaching.parking.local.parking_local_general.longConnection.util.SocketManager;
import com.huaching.parking.transfer.client.IClient;
import com.huaching.parking.transfer.codec.CustomLengthFrameDecoder;
import io.netty.channel.Channel;
import io.netty.channel.ChannelInitializer;
import io.netty.handler.codec.LengthFieldPrepender;
import io.netty.handler.timeout.IdleStateHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.annotation.Order;
import org.springframework.core.task.TaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.concurrent.TimeUnit;

/**
 * 与云服务的tcp长连接服务
 * Created by xiajl on 2017/7/28.
 */
@Component
@Order(1)
public class CloudConnectRunner implements CommandLineRunner {

    private Logger logger = LoggerFactory.getLogger(CloudConnectRunner.class);

    @Autowired
    private CloudConnectMessageProcess listener;

    @Value("${cloud.tcp.port}")
    int subPort;

    @Value("${cloud.tcp.host}")
    String subHost;

    @Value("${cloud.tcp.client.id}")
    String clientId;

    @Autowired
    protected TaskExecutor threadPool;

    private IClient client;

    @Override
    public void run(String... args) throws Exception {
        if(!StringUtils.hasText(clientId)){
            logger.error("未配置 cloud.tcp.client.id");
            return;
        }
        connect();
    }


    public void connect() {
        if(client == null){
            final AbstractLocalHandler handler = new CloudConnectHandler();
            handler.setThreadPool(threadPool);
            ChannelInitializer channelInitializer = new ChannelInitializer() {
                @Override
                protected void initChannel(Channel ch) throws Exception {
                    ch.pipeline().addLast("timeout", new IdleStateHandler(10, 10, 10, TimeUnit.SECONDS));
                    ch.pipeline().addLast(new LengthFieldPrepender(4, false));
                    ch.pipeline().addLast(new CustomLengthFrameDecoder(Integer.MAX_VALUE, 0, 4, 0, 4));
                    ch.pipeline().addLast(handler);
                }
            };
            SocketManager.CLOUD_TCP_CLIENT = clientId;
            client = new NettyClient(subHost,subPort, clientId,channelInitializer,true);
            handler.setClient(client);
            handler.addListener(listener);
//            handler.addLocalListener(listener);
            client.start();
        }
    }
}
