package com.huaching.parking.local.parking_local_general.service.local;

import com.alibaba.fastjson.JSON;
import com.huaching.parking.local.parking_local_general.domain.ResponseResult;
import com.huaching.parking.local.parking_local_general.domain.local.bo.ThsRequestChargeBO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ManualCarOutDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsCarInDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsCarOutDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsPaySyncDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.response.ThsRequestChargeDTO;
import com.huaching.parking.local.parking_local_general.domain.local.po.*;
import com.huaching.parking.local.parking_local_general.enums.local.ReasonCodeEnums;
import com.huaching.parking.local.parking_local_general.intercept.local.GeneralPrintLogAOP;
import com.huaching.parking.local.parking_local_general.service.local.base.*;
import com.huaching.parking.local.parking_local_general.service.zhaf.RemoteRequestService;
import com.huaching.parking.local.parking_local_general.util.local.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @program: parking_local_general
 * @description:
 * @author: lidongsheng
 * @createData: 2020-04-17 14:12
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-17 14:12
 * @updateContent:
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Service
public class LocalCarInAndOutServiceImpl implements LocalCarInAndOutService {

    private static final Logger logger = LoggerFactory.getLogger(LocalCarInAndOutServiceImpl.class);

    @Autowired
    private ChannelMessageService channelMessageService;

    @Autowired
    private ParkingRecordsService parkingRecordsService;

    @Autowired
    private CarAdvanceInOutRecordsService carAdvanceInOutRecordsService;

    @Autowired
    private ChargingRulesService chargingRulesService;

    @Autowired
    private ControlBarrierGateService controlBarrierGateService;

    @Autowired
    private PaymentChargesService paymentChargesService;

    @Autowired
    private ThsRequestChargeBO thsRequestChargeBO;

    @Autowired
    private PayOrderRecordsService payOrderRecordsService;

    @Override
    public ResponseResult carIn(ThsCarInDTO thsCarInDTO) {
        if (parkingRecordsService.add(ThsCarInDTO.dtoConvertToPo(thsCarInDTO))) {
            logger.info("添加入场记录成功");
            return ResponseResult.responseOK();
        } else {
            return ResponseResult.responseFail(ReasonCodeEnums.DB_OPERATION_ERROR, "添加入场记录失败");
        }
    }

    /**
     * 车辆出场上报
     *
     * @param thsCarOutDTO
     * @return
     */
    @Override
    public ResponseResult carOut(ThsCarOutDTO thsCarOutDTO) {
        try {
            ThsChannelMessagePO channelMessagePO = channelMessageService.getChannelMessage(thsCarOutDTO.getParkCode(), thsCarOutDTO.getOutChannelCode());
            if (channelMessagePO == null) {
                logger.error("未找到对应的通道信息，parkCode -> {} , channelCode -> {} ,", thsCarOutDTO.getParkCode(), thsCarOutDTO.getOutChannelCode());
                return ResponseResult.responseFail(ReasonCodeEnums.NO_PARK_CHANNEL_CONFIG, "未找到对应的通道信息");
            }
            String parkCode = thsCarOutDTO.getParkCode();
            String carNo = thsCarOutDTO.getCarNo();
            ThsParkingRecordsPO thsParkingRecordsPO = parkingRecordsService.getOnTheParkingRecord(parkCode, carNo);
            if (thsParkingRecordsPO == null) {
                logger.error("未查询到 parkCode -> {},carNo ->{} 有在场记录!", parkCode, carNo);
                return ResponseResult.responseFail(ReasonCodeEnums.NO_CAR_IN, "没有在场记录");
            }
            if (this.carOut(ThsCarOutDTO.dtoConvertToPo(thsCarOutDTO, thsParkingRecordsPO), 0, 0) == 0) {
                return ResponseResult.responseOK();
            } else {
                return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "出场逻辑处理失败");
            }
        } catch (Exception ex) {
            logger.error("车辆出场出异常：parkCode -> {} , carNo -> {}, ex -> {}", thsCarOutDTO.getParkCode(), thsCarOutDTO.getCarNo(), ex);
            return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "未知错误");
        }
    }

    @Override
    public ResponseResult carAdvanceInOut(ThsCarAdvanceInOutRecordsPO thsCarAdvanceInOutRecordsPO) {

        String parkCode = thsCarAdvanceInOutRecordsPO.getParkCode();
        String channelCode = thsCarAdvanceInOutRecordsPO.getCapChannelCode();
        String carNo = thsCarAdvanceInOutRecordsPO.getCarNo();
        if (thsCarAdvanceInOutRecordsPO.getCapFlag() == 1) {
            try {
                thsCarAdvanceInOutRecordsPO.setStatus(1);
                carAdvanceInOutRecordsService.add(thsCarAdvanceInOutRecordsPO);
            } catch (Exception ex) {
                logger.error("更新出场记录失败", ex);
                return ResponseResult.responseFail(ReasonCodeEnums.DB_OPERATION_ERROR, "更新出场记录失败");
            }
        } else {
            try {
                ThsParkingRecordsPO thsParkingRecordsPO = parkingRecordsService.getOnTheParkingRecord(parkCode, carNo);
                if (thsParkingRecordsPO == null) {
                    logger.error("未查询到 parkCode -> {},carNo ->{} 有在场记录!", parkCode, carNo);
                    return ResponseResult.responseFail(ReasonCodeEnums.NO_CAR_IN, "没有在场记录");
                }
                thsParkingRecordsPO = this.caioPoConvertToPrPo(thsCarAdvanceInOutRecordsPO, thsParkingRecordsPO);
                int result = this.carOut(thsParkingRecordsPO, Integer.MAX_VALUE, 1);
                logger.info("出场逻辑处理结果 parkCode -> {},carNo -> {},result -> {}", parkCode, carNo, result);
                thsCarAdvanceInOutRecordsPO.setStatus(result);
                if (carAdvanceInOutRecordsService.add(thsCarAdvanceInOutRecordsPO)) {
                    return ResponseResult.responseOK();
                } else {
                    return ResponseResult.responseFail(ReasonCodeEnums.DB_OPERATION_ERROR, "添加待出场记录失败");
                }
            } catch (Exception ex) {
                logger.error("车辆预出场处理出异常：parkCode -> {} ,channelCode -> {}, carNo -> {}, ex -> {}", parkCode, channelCode, carNo, ex);
                return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "车辆预出场处理出异常");
            }
        }
        return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "车辆预出场处理出异常");
    }

    /**
     * 预出场po转为ThsParkingRecordsPO
     *
     * @return
     */
    private ThsParkingRecordsPO caioPoConvertToPrPo(ThsCarAdvanceInOutRecordsPO thsCarAdvanceInOutRecordsPO, ThsParkingRecordsPO thsParkingRecordsPO) {

        thsParkingRecordsPO.setOutChannelCode(thsCarAdvanceInOutRecordsPO.getCapChannelCode());
        thsParkingRecordsPO.setOutImgUrl(thsCarAdvanceInOutRecordsPO.getCapImgUrl());
        thsParkingRecordsPO.setOnParkStatus(0);
        thsParkingRecordsPO.setUpdateTime(new Date());
        return thsParkingRecordsPO;
    }

    /**
     * 出场逻辑处理
     * 1.检查是否费用缴清
     * 2.更新数据库
     *
     * @param thsParkingRecordsPO
     * @param payableMoney        0标识无需支付
     * @return 0 开闸出场 1 费用未缴清依然在场  2开闸失败  3 数据库更新出场记录失败
     */
    @Override
    public int carOut(ThsParkingRecordsPO thsParkingRecordsPO, Integer payableMoney, Integer controlType) {
        String parkCode = thsParkingRecordsPO.getParkCode();
        String carNo = thsParkingRecordsPO.getCarNo();
        int result = 1;
        if (payableMoney == 0) {
            thsParkingRecordsPO.setPayUpStatus(1);
            thsParkingRecordsPO.setTotalMoney(0);
        } else {
            Integer totalMoney = getTotalMoney(thsParkingRecordsPO);
            totalMoney = totalMoney < 0 ? 0 : totalMoney;
            if (totalMoney == null) {
                totalMoney = Integer.MAX_VALUE;
            }

            ThsPayOrderRecordsPO thsPayOrderRecordsPO = payOrderRecordsService.selectLatestRecordByPrId(thsParkingRecordsPO.getId(), parkCode, carNo, 1);
            //用户费用缴清之后15分钟内免费出场，不计费
            if (totalMoney != 0
                    && (thsPayOrderRecordsPO != null && thsPayOrderRecordsPO.getPayedMoney().equals(thsPayOrderRecordsPO.getPayableMoney()))
                    && (thsPayOrderRecordsPO != null && DateUtils.subDate(thsPayOrderRecordsPO.getPayTime(), new Date(), 2) <= 15)) {
                logger.info("parkCode -> {},carNo ->{} 支付后15分钟内免费出场生效", thsParkingRecordsPO.getParkCode(), thsParkingRecordsPO.getCarNo());
                payableMoney = 0;
                thsParkingRecordsPO.setTotalMoney(thsParkingRecordsPO.getPayedMoney());
            } else {
                payableMoney = totalMoney - thsParkingRecordsPO.getPayedMoney();
            }

            logger.info("parkCode -> {},carNo ->{} 应付费用为 payableMoney -> {}：",
                    thsParkingRecordsPO.getParkCode(), thsParkingRecordsPO.getCarNo(), payableMoney);
            thsParkingRecordsPO.setPayUpStatus(payableMoney == 0 ? 1 : 0);
            thsParkingRecordsPO.setTotalMoney(totalMoney == Integer.MAX_VALUE ? thsParkingRecordsPO.getTotalMoney() : totalMoney);
        }
        boolean flag = false;
        boolean sendEventPublish = false;
        if (payableMoney == 0) {
            if (controlType == 1) {
                result = 2;
                if (controlBarrierGateService.controlBarrierGate(parkCode, thsParkingRecordsPO.getOutChannelCode(), 1)) {
                    logger.info("抬杆成功 parkCode -> {},carNo ->{},OutChannelCode -> {}：",
                            thsParkingRecordsPO.getParkCode(), thsParkingRecordsPO.getCarNo(), thsParkingRecordsPO.getOutChannelCode());
                    flag = true;
                    result = 0;
                } else {
                    logger.error("抬杆失败 parkCode -> {},carNo ->{},OutChannelCode -> {}：",
                            thsParkingRecordsPO.getParkCode(), thsParkingRecordsPO.getCarNo(), thsParkingRecordsPO.getOutChannelCode());
                }
            }
        }
        thsParkingRecordsPO.setOnParkStatus(1);
        if (flag) {
            thsParkingRecordsPO.setOnParkStatus(flag ? 0 : 2);
            thsParkingRecordsPO.setOutTime(new Date());
            sendEventPublish = true;
        }
        if (!parkingRecordsService.update(thsParkingRecordsPO, sendEventPublish)) {
            logger.error("数据库更新出场记录失败 parkCode -> {},carNo ->{},OutChannelCode -> {}：", thsParkingRecordsPO.getParkCode(), thsParkingRecordsPO.getCarNo(), thsParkingRecordsPO.getOutChannelCode());
            result = 3;
        }
        return result;
    }


    /**
     * 人工收费（车必须在场内（必须提供车牌号）或者是车已经到了出口）
     * 1. 检查是否有对应记录
     * 2. 支付同步
     *
     * @param manualCarOutDTO
     * @return
     */
    @Override
    public ResponseResult manualCharge(ManualCarOutDTO manualCarOutDTO) {
        String parkCode = manualCarOutDTO.getParkCode();
        String channelCode = manualCarOutDTO.getChannelNo();
        String carNo = manualCarOutDTO.getCarNo();
        if (carNo == null) {
            ThsCarAdvanceInOutRecordsPO thsCarAdvanceInOutRecordsPO = carAdvanceInOutRecordsService.selectLastByParkCodeAndChannelCode(parkCode, channelCode);
            carNo = thsCarAdvanceInOutRecordsPO.getCarNo();
            if (thsCarAdvanceInOutRecordsPO == null) {
                logger.error("未检测到 该车场(parkCode) -> {} 车场的 通道(channelCode) -> {} 下存在待出场车辆。", parkCode, channelCode);
                return ResponseResult.responseFail(ReasonCodeEnums.DB_OPERATION_ERROR, "未检测到存在待出场车辆，开闸失败");
            }
        }
        ThsRequestChargeDTO thsRequestChargeDTO = paymentChargesService.requestCharge(parkCode, carNo, new Date());
        ThsPaySyncDTO thsPaySyncDTO = new ThsPaySyncDTO();
        thsPaySyncDTO.setParkCode(parkCode);
        thsPaySyncDTO.setCarNo(carNo);
        thsPaySyncDTO.setPayedMoney(manualCarOutDTO.getPayedMoney());
        thsPaySyncDTO.setCouponMoney(thsRequestChargeDTO.getPayableMoney() - manualCarOutDTO.getPayedMoney());
        thsPaySyncDTO.setTerminalType(3);
        thsPaySyncDTO.setPayType(3);
        thsPaySyncDTO.setPayTime(new Date());
        thsPaySyncDTO.setOperatorUser(manualCarOutDTO.getOperatorUser());
        thsPaySyncDTO.setRemake(manualCarOutDTO.getRemake());
        return paymentChargesService.paySync(thsPaySyncDTO);
    }

    private Integer getTotalMoney(ThsParkingRecordsPO thsParkingRecordsPO) {
        String parkCode = thsParkingRecordsPO.getParkCode();
        String carNo = thsParkingRecordsPO.getCarNo();
        try {
            ThsChargingRulesPO thsChargingRulesPO = chargingRulesService.find(parkCode);
            if (thsChargingRulesPO == null) {
                logger.error("未查询到 parkCode -> {} 配置收费规则", parkCode);
                return null;
            }
            return thsRequestChargeBO.getPayableMoney(thsChargingRulesPO, carNo, thsParkingRecordsPO.getInTime());
        } catch (Exception ex) {
            logger.error("出场请求费用出错， parkCode -> {} ，carNo -> {}", parkCode, carNo);
            return null;
        }
    }

}
