package com.huaching.parking.local.parking_local_general.service.local;

import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsMemberCarDTO;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsMemberCarPO;
import com.huaching.parking.local.parking_local_general.mapper.local.ThsMemberCarPOMapper;
import com.huaching.parking.local.parking_local_general.service.local.base.MemeberCarService;
import com.huaching.parking.local.parking_local_general.util.local.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Calendar;
import java.util.Date;

/**
 * @program: parking_local_general
 * @description:
 * @author: lidongsheng
 * @createData: 2020-04-29 11:16
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-29 11:16
 * @updateContent:
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Service
public class MemberCarHelperServiceImpl implements MemberCarHelperService {

    private static final Logger logger = LoggerFactory.getLogger(MemberCarHelperServiceImpl.class);

    @Autowired
    private MemeberCarService memeberCarService;

    @Override
    public boolean memberSync(ThsMemberCarDTO thsMemberCarDTO) {
        logger.info("会员权益下发初始参数，dto->{}", thsMemberCarDTO);
        ThsMemberCarPO thsMemberCarPO = memeberCarService.selectMemberCarDataInfo(thsMemberCarDTO.getParkCode(), thsMemberCarDTO.getCarNo(), 1);
        //新增会员
        if (thsMemberCarPO == null) {
            logger.info("进行新增会员操作");
            thsMemberCarPO = new ThsMemberCarPO();
            thsMemberCarPO.setCarNo(thsMemberCarDTO.getCarNo());
            thsMemberCarPO.setStartDate(thsMemberCarDTO.getStartDate());
            thsMemberCarPO.setEndDate(thsMemberCarDTO.getEndDate());
            thsMemberCarPO.setCarType(thsMemberCarDTO.getCarType());
            switch (thsMemberCarDTO.getCarType()) {
                case 1:
                    thsMemberCarPO.setBenefitsMoney(0);
                    thsMemberCarPO.setBenefitsTime(0);
                    break;
                case 2:
                    thsMemberCarPO.setBenefitsTime(thsMemberCarDTO.getBenefitsTime());
                    thsMemberCarPO.setBenefitsMoney(0);
                    break;
                case 3:
                    thsMemberCarPO.setBenefitsMoney(thsMemberCarDTO.getBenefitsMoney());
                    thsMemberCarPO.setBenefitsTime(0);
                    break;
            }
            //权益详情备注字段
            thsMemberCarPO.setBenefitsRemark(thsMemberCarDTO.getBenefitsRemark());
            thsMemberCarPO.setCreatTime(new Date());
            thsMemberCarPO.setUpdateTime(new Date());
            thsMemberCarPO.setStatus(1);
            thsMemberCarPO.setParkCode(thsMemberCarDTO.getParkCode());
            thsMemberCarPO.setMemberName(thsMemberCarDTO.getMemberName());
            thsMemberCarPO.setMemberPhone(thsMemberCarDTO.getMemberPhone());
            thsMemberCarPO.setRemake(thsMemberCarDTO.getRemake());
            return memeberCarService.add(thsMemberCarPO);
        } else {
            logger.info("进行会员续费操作");
            //会员续费
            Date now = new Date();
            Date oldEndDate = thsMemberCarPO.getEndDate();
            if (!now.after(oldEndDate)) {
                //会员未过期
                int day = (int) DateUtils.subDate(thsMemberCarDTO.getStartDate(), thsMemberCarDTO.getEndDate(), 4);
                Date newEndDate = DateUtils.calAddDate(oldEndDate, day, Calendar.DAY_OF_YEAR);
                thsMemberCarPO.setEndDate(newEndDate);
            } else {
                thsMemberCarPO.setStartDate(thsMemberCarDTO.getStartDate());
                thsMemberCarPO.setEndDate(thsMemberCarDTO.getEndDate());
            }
            thsMemberCarPO.setUpdateTime(new Date());
            return memeberCarService.update(thsMemberCarPO);
        }
    }

    @Override
    public boolean memberDel(String parkCode, String carNo) {
        logger.info("会员权益删除，parkCode -> {} , carNo -> {}", parkCode, carNo);
        return memeberCarService.delete(parkCode, carNo);
    }

    @Override
    public ThsMemberCarPO vaildMember(String parkCode, String carNo) {
        ThsMemberCarPO thsMemberCarPO = memeberCarService.selectMemberCarDataInfo(parkCode, carNo, 1);
        Date now = new Date();
        //是否为会员并且会员未过期
        if (thsMemberCarPO != null) {
            if (!now.after(thsMemberCarPO.getEndDate())) {
                return thsMemberCarPO;
            } else {
                memeberCarService.delete(parkCode, carNo);
            }
        }
        return null;
    }
}
