package com.huaching.parking.local.parking_local_general.service.local;

import com.alibaba.fastjson.JSON;
import com.huaching.parking.local.parking_local_general.domain.ResponseResult;
import com.huaching.parking.local.parking_local_general.domain.local.bo.ThsRequestChargeBO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.request.ThsPaySyncDTO;
import com.huaching.parking.local.parking_local_general.domain.local.dto.response.ThsRequestChargeDTO;
import com.huaching.parking.local.parking_local_general.domain.local.po.*;
import com.huaching.parking.local.parking_local_general.enums.local.ReasonCodeEnums;
import com.huaching.parking.local.parking_local_general.factory.ThreadPoolFactory;
import com.huaching.parking.local.parking_local_general.intercept.local.GeneralPrintLogAOP;
import com.huaching.parking.local.parking_local_general.service.local.base.*;
import com.huaching.parking.local.parking_local_general.util.local.DateUtils;
import com.huaching.parking.local.parking_local_general.util.local.chargeCalculator.ChargeCalculatorUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @program: parking_local_general
 * @description:
 * @author: lidongsheng
 * @createData: 2020-04-22 18:50
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-22 18:50
 * @updateContent:
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Service
public class PaymentChargesServiceImpl implements PaymentChargesService {

    private static final Logger logger = LoggerFactory.getLogger(PaymentChargesServiceImpl.class);

    @Autowired
    private ParkingRecordsService parkingRecordsService;

    @Autowired
    private ChargingRulesService chargingRulesService;

    @Autowired
    private PayOrderRecordsService payOrderRecordsService;

    @Autowired
    private CarAdvanceInOutRecordsService carAdvanceInOutRecordsService;

    @Autowired
    private LocalCarInAndOutService localCarInAndOutService;

    @Autowired
    private ThsRequestChargeBO thsRequestChargeBO;

    @Autowired
    private ChargeCalculatorUtil chargeCalculatorUtil;

    @Autowired
    private ParkMessageService parkMessageService;

    @Override
    @GeneralPrintLogAOP(value = "场内付请求费用", jsonSubs = {-1})
    public ThsRequestChargeDTO requestCharge(String parkCode, String carNo, Date outTime) {
        Integer thsParkingRecordsId;
        String orderNo;
        Integer totalMoney, payableMoney, tempPayableMoney;
        ThsParkingRecordsPO thsParkingRecordsPO;
        try {
            thsParkingRecordsPO = parkingRecordsService.getOnTheParkingRecord(parkCode, carNo);
            if (thsParkingRecordsPO == null || (outTime != null && thsParkingRecordsPO.getInTime().after(outTime))) {
                logger.error("未查询到 parkCode -> {},carNo ->{} 有在场记录!", parkCode, carNo);
//                return ResponseResult.responseFail(ReasonCodeEnums.NO_CAR_IN, "未查询到有在场记录");
                return ThsRequestChargeDTO.errorMsg("未检测到在场记录");
            }
            ThsChargingRulesPO thsChargingRulesPO = chargingRulesService.find(parkCode);
            if (thsChargingRulesPO == null) {
                logger.error("未查询到 parkCode -> {} 配置收费规则", parkCode);
//                return ResponseResult.responseFail(ReasonCodeEnums.NO_PARK_CHARGE_CONFIG, "该车场未配置收费规则");
                return ThsRequestChargeDTO.errorMsg("该车场未配置收费规则");
            }
            thsParkingRecordsId = thsParkingRecordsPO.getId();
            payableMoney = thsRequestChargeBO.getPayableMoney(thsChargingRulesPO, carNo, thsParkingRecordsPO.getInTime());
            totalMoney = payableMoney < 0 ? 0 : payableMoney;
            payableMoney -= thsParkingRecordsPO.getPayedMoney();
            payableMoney = payableMoney < 0 ? 0 : payableMoney;
            tempPayableMoney = payableMoney;
            logger.info("parkCode -> {}, carNo ->{}, 总计停车费用为 ->{} ,应收停车费用为 ->{}", parkCode, carNo, totalMoney, payableMoney);
            orderNo = this.creatOrderNo(parkCode, carNo);
        } catch (Exception ex) {
            logger.error("请求费用异常 ex -> {}", ex.toString());
            return ThsRequestChargeDTO.errorMsg("请求费用异常");
//            return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "请求费用异常，请重试");
        }
        thsParkingRecordsPO.setTotalMoney(totalMoney);
        ThreadPoolFactory.getGeneral().execute(() -> addPayOrderRecords(parkCode, carNo, thsParkingRecordsId, orderNo, tempPayableMoney, thsParkingRecordsPO));
        ThsRequestChargeDTO result = ThsRequestChargeDTO.poConvertToDto(thsParkingRecordsPO, orderNo,
                payableMoney);
        logger.info("请求费用完成 parkCode -> {},carNo ->{}", parkCode, carNo);
        return result;
//        return ResponseResult.Builder.init().setData(JSON.toJSONString(result)).build();
    }

    @Override
    public ThsRequestChargeDTO requestChargeOld(String parkCode, String carNo) {
        Integer thsParkingRecordsId;
        String orderNo;
        ThsParkingRecordsPO thsParkingRecordsPO;
        Integer totalMoney, payableMoney, tempPayableMoney;
        try {

            thsParkingRecordsPO = parkingRecordsService.getOnTheParkingRecord(parkCode, carNo);
            thsParkingRecordsId = thsParkingRecordsPO.getId();
            if (thsParkingRecordsPO == null) {
                logger.error("未查询到 parkCode -> {},carNo ->{} 有在场记录!", parkCode, carNo);
//                return ResponseResult.responseFail(ReasonCodeEnums.NO_CAR_IN, "未查询到有在场记录");
                return ThsRequestChargeDTO.errorMsg("未检测到在场记录");
            }
            ThsParkMessagePO thsParkMessagePO = parkMessageService.getByParkCode(parkCode);
            Date endTime = thsParkingRecordsPO.getOutTime() == null ? new Date() : thsParkingRecordsPO.getOutTime();
            payableMoney = chargeCalculatorUtil.getChargeInfo(thsParkingRecordsPO.getInTime(), endTime, Integer.valueOf(parkCode),
                    thsParkMessagePO.getChargeType().byteValue(), thsParkMessagePO.getFreeStatus() == 0 ? false : true);
            totalMoney = payableMoney < 0 ? 0 : payableMoney;
            payableMoney -= thsParkingRecordsPO.getPayedMoney();
            tempPayableMoney = payableMoney;
            logger.info("parkCode -> {}, carNo ->{}, 总计停车费用为 ->{} ,应收停车费用为 ->{}", parkCode, carNo, totalMoney, payableMoney);
            orderNo = this.creatOrderNo(parkCode, carNo);
        } catch (Exception ex) {
            logger.error("请求费用异常 ex -> {}", ex.toString());
            return ThsRequestChargeDTO.errorMsg("请求费用异常");
//            return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "请求费用异常，请重试");
        }
        thsParkingRecordsPO.setTotalMoney(totalMoney);
        ThreadPoolFactory.getGeneral().execute(() -> addPayOrderRecords(parkCode, carNo, thsParkingRecordsId, orderNo, tempPayableMoney, thsParkingRecordsPO));
        ThsRequestChargeDTO result = ThsRequestChargeDTO.poConvertToDto(thsParkingRecordsPO, orderNo,
                payableMoney);
        logger.info("请求费用完成 parkCode -> {},carNo ->{}", parkCode, carNo);
        return result;
//        return ResponseResult.Builder.init().setData(JSON.toJSONString(result)).build();
    }

    @Override
    @GeneralPrintLogAOP(value = "出口付请求费用", jsonSubs = {-1})
    public ThsRequestChargeDTO outRequestCharge(String parkCode, String channelCode) {
        ThsCarAdvanceInOutRecordsPO thsCarAdvanceInOutRecordsPO = carAdvanceInOutRecordsService.selectLastByParkCodeAndChannelCode(parkCode, channelCode);
        if (thsCarAdvanceInOutRecordsPO == null) {
            logger.error("未查询到该车辆待出场记录，parkCode -> {}, channelCode -> {}", parkCode, channelCode);
//            return ResponseResult.responseFail(ReasonCodeEnums.NO_ADVANCE_OUT_RECORDS, "未查询到待出场订单记录");
            return ThsRequestChargeDTO.errorMsg("未查询到该车辆待出场记录");
        }
        ThsRequestChargeDTO thsRequestChargeDTO = this.requestCharge(thsCarAdvanceInOutRecordsPO.getParkCode(), thsCarAdvanceInOutRecordsPO.getCarNo(), thsCarAdvanceInOutRecordsPO.getCapTime());
        thsRequestChargeDTO.setOutTime(thsCarAdvanceInOutRecordsPO.getCapTime());
        return thsRequestChargeDTO;
    }


    //支付同步要判断是场内付还是出口付

    @Override
    @Transactional
    @GeneralPrintLogAOP(value = "支付同步")
    public ResponseResult paySync(ThsPaySyncDTO thsPaySyncDTO) {

        ThsPayOrderRecordsPO thsPayOrderRecordsPO = payOrderRecordsService.selectLatestRecord(
                thsPaySyncDTO.getParkCode(), thsPaySyncDTO.getCarNo(), 0);

        if (thsPayOrderRecordsPO == null) {
            logger.error("未查询到待支付订单记录，parkCode -> {}, carNo -> {}", thsPaySyncDTO.getParkCode(), thsPaySyncDTO.getCarNo());
            return ResponseResult.responseFail(ReasonCodeEnums.NO_UNPAID_PAY_ORDER_RECODE, "支付同步未查询到待支付订单记录");
//            return ResponseResult.responseFail(ReasonCodeEnums.NO_UNPAID_PAY_ORDER_RECODE, "未查询到待支付订单记录，请尝试重新请求费用");
        }
//
//        if (!thsPayOrderRecordsPO.getOrderNo().equals(thsPaySyncDTO.getOrderNo())) {
//            logger.error("orderNo 已经过期或者不存在，parkCode -> {}, carNo -> {},paramOrderNo -> {}, DBOrderNo -> {}",
//                    thsPaySyncDTO.getParkCode(), thsPaySyncDTO.getCarNo(), thsPaySyncDTO.getOrderNo(), thsPayOrderRecordsPO.getOrderNo());
//            return false;
////            return ResponseResult.responseFail(ReasonCodeEnums.ORDER_NO_IS_EXPIRE, "orderNo 已经过期或者不存在，请尝试重新请求费用获得新的orderNo");
//        }
        try {
            ThsParkingRecordsPO thsParkingRecordsPO = parkingRecordsService.selectById(thsPayOrderRecordsPO.getParkingRecordsId());
            thsParkingRecordsPO.setPayedMoney(thsParkingRecordsPO.getPayedMoney() + thsPaySyncDTO.getPayedMoney());

            thsPayOrderRecordsPO.setPayedMoney(thsPaySyncDTO.getPayedMoney());
            thsPayOrderRecordsPO.setPayTime(thsPaySyncDTO.getPayTime());
            thsPayOrderRecordsPO.setPayType(thsPaySyncDTO.getPayType());
            thsPayOrderRecordsPO.setTerminalType(thsPaySyncDTO.getTerminalType());
            thsPayOrderRecordsPO.setUpdateTime(new Date());
            thsPayOrderRecordsPO.setPayStatus(1);
            thsPayOrderRecordsPO.setCouponMoney(thsPaySyncDTO.getCouponMoney());
            payOrderRecordsService.update(thsPayOrderRecordsPO);
            //判断是否为出口付
            ThsCarAdvanceInOutRecordsPO advanceInOutRecordsPO = carAdvanceInOutRecordsService
                    .selectLastAdvanceCarOutRecord(thsPaySyncDTO.getParkCode(), thsPaySyncDTO.getCarNo());
            if (advanceInOutRecordsPO != null && advanceInOutRecordsPO.getCapTime().after(thsParkingRecordsPO.getInTime())) {
                logger.info("请求费用 parkCode -> {}, carNo -> {} 检测到应该为出口付", thsPaySyncDTO.getParkCode(), thsPaySyncDTO.getCarNo());
                thsParkingRecordsPO.setOutChannelCode(advanceInOutRecordsPO.getCapChannelCode());
                Integer status = localCarInAndOutService.carOut(thsParkingRecordsPO, Integer.MAX_VALUE, 1);
                advanceInOutRecordsPO.setStatus(status);
                carAdvanceInOutRecordsService.update(advanceInOutRecordsPO);
                switch (status) {
                    case 0:
                        return ResponseResult.responseOK();
                    case 1:
                        return ResponseResult.responseFail(ReasonCodeEnums.CHARGES_NOT_PAID, "费用未缴清，请尝试重新获取费用");
                    case 2:
                        return ResponseResult.responseFail(ReasonCodeEnums.CONTROL_BARRIER_GATE, "控制道闸失败");
                    default:
                        return ResponseResult.responseFail(ReasonCodeEnums.DB_OPERATION_ERROR, "更新出场记录失败");
                }
            } else {
                parkingRecordsService.update(thsParkingRecordsPO, true);
            }
            logger.info("支付同步完成");
            return ResponseResult.responseOK();
//            return ResponseResult.responseOK();
        } catch (Exception ex) {
            logger.error("支付同步 异常 ex ->", ex);
            return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "支付同步异常，请重试");
//            return ResponseResult.responseFail(ReasonCodeEnums.UNKNOWN, "支付同步异常，请重试");
        }
    }

    private boolean addPayOrderRecords(String parkCode, String carNo, Integer parkingRecordId, String orderNo, Integer payableMoney, ThsParkingRecordsPO thsParkingRecordsPO) {
        logger.info("添加待支付订单记录以及更新停车记录的总费用,parkingRecordId -> {} , orderNo -> {} , payableMoney -> {}", parkingRecordId, orderNo, payableMoney);
        ThsPayOrderRecordsPO thsPayOrderRecordsPO = new ThsPayOrderRecordsPO();
        thsPayOrderRecordsPO.setParkingRecordsId(parkingRecordId);
        thsPayOrderRecordsPO.setOrderNo(orderNo);
        thsPayOrderRecordsPO.setPayedMoney(0);
        thsPayOrderRecordsPO.setCouponMoney(0);
        thsPayOrderRecordsPO.setPayableMoney(payableMoney);
        thsPayOrderRecordsPO.setPayStatus(0);
        thsPayOrderRecordsPO.setParkCode(parkCode);
        thsPayOrderRecordsPO.setCarNo(carNo);
        thsPayOrderRecordsPO.setCreatTime(new Date());
        if (parkingRecordsService.update(thsParkingRecordsPO, false) && payOrderRecordsService.add(thsPayOrderRecordsPO)) {
            return true;
        }
        return false;
    }

    private String creatOrderNo(String parkCode, String carNo) {
        String sole = parkCode + carNo + System.currentTimeMillis();
        String uuid = UUID.nameUUIDFromBytes(sole.getBytes()).toString();
        Integer code = uuid.hashCode() & Integer.MAX_VALUE;
        String oderNo = "DD" + code + uuid.substring(10, 12);
        logger.info("parkCode -> {},carNo ->{},自动生成唯一订单号为 -> {}", parkCode, carNo, oderNo);
        return oderNo;
    }
}
