package com.huaching.parking.local.parking_local_general.service.local.base.impl;

import com.alibaba.fastjson.JSON;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsParkingRecordsPO;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsParkingRecordsPOExample;
import com.huaching.parking.local.parking_local_general.longConnection.util.SocketManager;
import com.huaching.parking.local.parking_local_general.mapper.local.ThsParkingRecordsPOMapper;
import com.huaching.parking.local.parking_local_general.service.local.base.ParkingRecordsService;
import com.huaching.parking.transfer.domain.dto.PlatformCarInOutDto;
import com.huaching.parking.transfer.domain.pojo.EventPublish;
import com.huaching.parking.transfer.enums.PlatformDataInterfaceTypeEnums;
import com.huaching.parking.transfer.protocol.Request;
import com.huaching.parking.transfer.protocol.Response;
import com.huaching.parking.transfer.service.EventPublishService;
import com.huaching.parking.transfer.util.SynchUtil;
import io.netty.channel.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @program: parking_local_general
 * @description: 停车记录与数据库操作
 * @author: lidongsheng
 * @createData: 2020-04-21 16:04
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-21 16:04
 * @updateContent: 停车记录
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Service
public class ParkingRecordsServiceImpl implements ParkingRecordsService {

    private static final Logger logger = LoggerFactory.getLogger(ParkingRecordsServiceImpl.class);

    @Autowired
    private ThsParkingRecordsPOMapper mapper;

    @Autowired
    private EventPublishService eventPublishService;


    @Override
    public boolean add(ThsParkingRecordsPO thsParkingRecordsPO) {

        eventPublish(thsParkingRecordsPO, PlatformDataInterfaceTypeEnums.IN_PUSH.getType());
        return mapper.insert(thsParkingRecordsPO) == 1 ? true : false;
    }

    @Override
    public boolean deleteById(Integer id) {
        return mapper.deleteByPrimaryKey(id) == 1 ? true : false;
    }

    @Override
    public int deleteByExample(ThsParkingRecordsPO thsParkingRecordsPO) {
        ThsParkingRecordsPOExample example = new ThsParkingRecordsPOExample();
        ThsParkingRecordsPOExample.Criteria criteria = example.createCriteria();
        if (thsParkingRecordsPO.getParkCode() == null || thsParkingRecordsPO.getCarNo() == null) {
            return -1;
        }
        criteria.andParkCodeEqualTo(thsParkingRecordsPO.getParkCode())
                .andCarNoEqualTo(thsParkingRecordsPO.getCarNo());
        if (thsParkingRecordsPO.getInTime() != null) {
            criteria.andInTimeEqualTo(thsParkingRecordsPO.getInTime());
        }
        if (thsParkingRecordsPO.getOutTime() != null) {
            criteria.andOutTimeEqualTo(thsParkingRecordsPO.getOutTime());
        }
        if (thsParkingRecordsPO.getInChannelCode() != null) {
            criteria.andInChannelCodeEqualTo(thsParkingRecordsPO.getInChannelCode());
        }
        if (thsParkingRecordsPO.getOutChannelCode() != null) {
            criteria.andOutChannelCodeEqualTo(thsParkingRecordsPO.getOutChannelCode());
        }
        if (thsParkingRecordsPO.getOnParkStatus() != null) {
            criteria.andOnParkStatusEqualTo(thsParkingRecordsPO.getOnParkStatus());
        }
        return mapper.deleteByExample(example);
    }

    @Override
    public boolean update(ThsParkingRecordsPO thsParkingRecordsPO, boolean sendEventPublish) {
        if (sendEventPublish) {
            eventPublish(thsParkingRecordsPO, PlatformDataInterfaceTypeEnums.OUT_PUSH.getType());
        }
        ThsParkingRecordsPOExample example = new ThsParkingRecordsPOExample();
        ThsParkingRecordsPOExample.Criteria criteria = example.createCriteria();
        if (thsParkingRecordsPO.getParkCode() == null || thsParkingRecordsPO.getCarNo() == null || thsParkingRecordsPO.getInTime() == null) {
            return false;
        }
        criteria.andParkCodeEqualTo(thsParkingRecordsPO.getParkCode())
                .andCarNoEqualTo(thsParkingRecordsPO.getCarNo())
                .andInTimeEqualTo(thsParkingRecordsPO.getInTime());
        return mapper.updateByExampleSelective(thsParkingRecordsPO, example) == 1 ? true : false;
    }

    @Override
    public ThsParkingRecordsPO getOnTheParkingRecord(String parkCode, String carNo) {
        return mapper.getOnTheParkingRecord(parkCode, carNo);
    }

    @Override
    public ThsParkingRecordsPO selectById(Integer id) {
        return mapper.selectByPrimaryKey(id);
    }


    private void eventPublish(ThsParkingRecordsPO thsParkingRecordsPO, String kind) {
        PlatformCarInOutDto vo = toPlatformDto(thsParkingRecordsPO);
        EventPublish publish = SynchUtil.transferToPublishImmediate(JSON.toJSONString(vo), kind, SocketManager.CLOUD_TCP_CLIENT);

        //1、先发送远程同步数据
        Request request = SynchUtil.coventFromPublishImmediateEvent(publish);
        Response response = null;
        Channel channel = SocketManager.getInstance().getChannelByModelName(SocketManager.CLOUD_TCP_CLIENT);
        if (null != channel && channel.isActive()) {
            try {
                response = SynchUtil.syncRequest(channel, request);
            } catch (Exception e) {
                e.printStackTrace();
                logger.error("发送远程同步数据失败");
            }
        }

        if (null != response && response.getCode() == 1) {
            publish.setEventStatus((byte) 1);
        } else {
            publish.setEventStatus((byte) 0);
        }
        //2、记录化发布数据操作
        eventPublishService.insertEventPublish(publish);
    }

    private PlatformCarInOutDto toPlatformDto(ThsParkingRecordsPO thsParkingRecordsPO) {
        PlatformCarInOutDto vo = new PlatformCarInOutDto();
        vo.setValue(thsParkingRecordsPO.getTotalMoney());
        vo.setParkId(Integer.valueOf(thsParkingRecordsPO.getParkCode()));
        vo.setCardType(0);
        vo.setCarNo(thsParkingRecordsPO.getCarNo());

        vo.setInTime(thsParkingRecordsPO.getInTime());
        vo.setOutTime(thsParkingRecordsPO.getOutTime());
        return vo;

    }
}
