package com.huaching.parking.local.parking_local_general.service.zhaf.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.HashMultimap;
import com.huaching.parking.local.parking_local_general.config.zhaf.DZConfig;
import com.huaching.parking.local.parking_local_general.domain.ResponseResult;
import com.huaching.parking.local.parking_local_general.domain.zhaf.ResponseResultZHAF;
import com.huaching.parking.local.parking_local_general.domain.zhaf.dto.BindDeviceDTO;
import com.huaching.parking.local.parking_local_general.domain.zhaf.dto.DeleteDeviceDTO;
import com.huaching.parking.local.parking_local_general.domain.zhaf.dto.RegisterDeviceDTO;
import com.huaching.parking.local.parking_local_general.domain.zhaf.dto.SubscriptionDTO;
import com.huaching.parking.local.parking_local_general.enums.local.ReasonCodeEnums;
import com.huaching.parking.local.parking_local_general.factory.ThreadPoolFactory;
import com.huaching.parking.local.parking_local_general.service.zhaf.RemoteRequestService;
import org.apache.http.client.ClientProtocolException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestClientResponseException;
import org.springframework.web.client.RestTemplate;

import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @program: parking_local_general
 * @description:
 * @author: lidongsheng
 * @createData: 2020-04-29 13:28
 * @updateAuthor: lidongsheng
 * @updateData: 2020-04-29 13:28
 * @updateContent:
 * @Version: 1.0.0

 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */
@Service
public class RemoteRequestServiceImpl implements RemoteRequestService {

    private static final Logger logger = LoggerFactory.getLogger(RemoteRequestServiceImpl.class);

    @Autowired
    private RestTemplate restTemplate;


    private static final String BARRIERGATE = "/lidig/control/barrierGate/";

    private static final String REGISTER_DEVICE = "/lidig/device";

    private static final String SUBSCRIPTION = "/lidig/subscription";

    private static final String DEVICE_BIND = "/lidig/deviceBind";

    private static final String CTRL_MODE = "/lidig/ctrlMode/";


    @Override
    public ResponseResultZHAF registerDevice(RegisterDeviceDTO registerDeviceDTO) {
        List<RegisterDeviceDTO> registerDeviceDTOS = new ArrayList<>();
        registerDeviceDTOS.add(registerDeviceDTO);
        return reqHolder(DZConfig.getAddress() + REGISTER_DEVICE, JSON.toJSONString(registerDeviceDTOS), HttpMethod.POST, MediaType.APPLICATION_JSON_UTF8);
    }

    @Override
    public ResponseResultZHAF deleteDevice(DeleteDeviceDTO deleteDeviceDTO) {
        return reqHolder(DZConfig.getAddress() + REGISTER_DEVICE, JSON.toJSONString(deleteDeviceDTO), HttpMethod.DELETE, MediaType.MULTIPART_FORM_DATA);
    }

    @Override
    public ResponseResultZHAF bindDevice(BindDeviceDTO bindDeviceDTO) {
        List<BindDeviceDTO> bindDeviceDTOS = new ArrayList<>();
        bindDeviceDTOS.add(bindDeviceDTO);
        return reqHolder(DZConfig.getAddress() + DEVICE_BIND, JSON.toJSONString(bindDeviceDTOS), HttpMethod.POST, MediaType.APPLICATION_JSON_UTF8);
    }

    @Override
    public ResponseResultZHAF subscription(SubscriptionDTO subscriptionDTO) {
        return reqHolder(DZConfig.getAddress() + SUBSCRIPTION, JSON.toJSONString(subscriptionDTO), HttpMethod.POST, MediaType.MULTIPART_FORM_DATA);
    }

    @Override
    public ResponseResultZHAF deleteSubscription(SubscriptionDTO subscriptionDTO) {
        return reqHolder(DZConfig.getAddress() + SUBSCRIPTION, JSON.toJSONString(subscriptionDTO), HttpMethod.DELETE, MediaType.MULTIPART_FORM_DATA);
    }

    @Override
    public ResponseResult controlBarrierGate(String deviceCode, Integer type) {

        ResponseResultZHAF result = reqHolder(DZConfig.getAddress() + BARRIERGATE + deviceCode + "/" + type, null, HttpMethod.POST, MediaType.APPLICATION_JSON_UTF8);
        if (result.isSuccess()) {
            return ResponseResult.responseOK();
        } else {
            return ResponseResult.responseFailPublic(result.getCode(), result.getMsg().toString());
        }
    }

    @Override
    public ResponseResultZHAF ctrlMode(String deviceCode, Integer type) {
        return reqHolder(DZConfig.getAddress() + CTRL_MODE + deviceCode + "/" + type, null, HttpMethod.PUT, MediaType.APPLICATION_JSON_UTF8);
    }


    /**
     * 统一转发请求
     *
     * @param url  请求地址
     * @param data data
     * @return
     */
    public ResponseResultZHAF reqHolder(String url, Object data, HttpMethod httpMethod, MediaType mediaType) {
        logger.error("统一下发请求，url:" + url + ",data:" + JSON.toJSONString(data));
        ResponseResultZHAF result = new ResponseResultZHAF();
        try {
            result.setCode(ReasonCodeEnums.UNKNOWN.getCode());
            result.setSuccess(false);
            try {
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(mediaType);
                ResponseEntity<ResponseResultZHAF> responseEntity = null;
                if (MediaType.APPLICATION_JSON_UTF8.equals(mediaType)) {
                    HttpEntity<String> requestEntity = null;
                    if (data != null) {
                        requestEntity = new HttpEntity<String>((String) data, headers);
                    }
                    responseEntity = restTemplate.exchange(url, httpMethod, requestEntity, ResponseResultZHAF.class);
                } else {
                    MultiValueMap<String, Object> multiValueMap = new LinkedMultiValueMap<>();
                    JSONObject jsonObject = JSONObject.parseObject((String) data);
                    for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
                        multiValueMap.add(entry.getKey(), entry.getValue());
                    }
                    HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<>(
                            multiValueMap, headers);
                    responseEntity = restTemplate.exchange(url, httpMethod, requestEntity, ResponseResultZHAF.class);
                }
                result = responseEntity.getBody();
                logger.info("统一转发请求成功，url:" + url + ",data:" + data);
                logger.info("统一转发请求返回数据，result: status" + responseEntity.getStatusCodeValue() + " ,data:" + JSON.toJSONString(result));
            } catch (ResourceAccessException rex) {
                Throwable throwable = rex.getCause();
                //判断是否为超时
                if (throwable.getCause() instanceof SocketTimeoutException) {
                    logger.error("统一转发请求执行任务超时, url->" + url);
                } else if (throwable.getCause() instanceof ClientProtocolException) {
                    logger.error("统一转发客户端规则（ClientProtocolException）请求出异常, url->" + url);
                } else {
                    logger.error("统一转发请求出异常, url -> " + url + ",ex:" + rex.getMessage());
                }
            } catch (RestClientResponseException e) {
                logger.error("统一转发请求出异常(RestClientResponseException)，url -> " + url);
                logger.error("异常信息，e:", e);
            } catch (Exception ex) {
                logger.error("统一转发请求出异常，url:" + url);
                logger.error("异常信息，ex:", ex);
            }
        } catch (Exception ex) {
            logger.error("统一转发请求线程池出异常，url:" + url);
            logger.error("异常信息，ex:" + ex.getMessage());
        }
        return result;
    }
}
