package com.huaching.parking.local.parking_local_general.util;

import com.alibaba.fastjson.JSON;
import org.apache.http.client.methods.*;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;

/**
 */
public class HttpRestfulUtil {
	private static Logger logger = LoggerFactory.getLogger(HttpRestfulUtil.class);

	/**
	 * post
	 * @param urlPath
	 * @param json
	 * @return
	 */
	public static String doJsonPost(String urlPath, String json,String user,String pwd) {

		CloseableHttpClient client = HttpClients.createDefault();
		HttpPost post = new HttpPost();

		try {
			post.setURI(new URI(urlPath));
			post.addHeader("Content-Type", "application/json;charset=utf-8");
			post.addHeader("user", user);
			post.addHeader("pwd", pwd);

			logger.error("doJsonPost:user->"+user+"  pwd->"+pwd);
			StringEntity entity = new StringEntity(json, "utf-8");
			post.setEntity(entity);
			CloseableHttpResponse resp = client.execute(post);
			if (resp.getStatusLine().getStatusCode() == 200) {
				logger.info("请求返回数据" + JSON.toJSONString(resp));
				return handleResponse(resp);
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("调用请求地址：" + urlPath + "失败！");
		}
		return null;
	}


	/**
	 * put
	 * @param urlPath
	 * @param json
	 * @return
	 */
	public static String doJsonPut(String urlPath, String json) {

		CloseableHttpClient client = HttpClients.createDefault();
		HttpPut httpPut = new HttpPut();

		try {

			httpPut.setURI(new URI(urlPath));
			httpPut.addHeader("Content-Type", "application/json");
			StringEntity entity = new StringEntity(json, "utf-8");
			httpPut.setEntity(entity);

			CloseableHttpResponse resp = client.execute(httpPut);
			if (resp.getStatusLine().getStatusCode() == 200) {
				logger.debug("请求返回数据" + JSON.toJSONString(resp));
				return handleResponse(resp);
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("调用请求地址：" + urlPath + "失败！");
		}
		return null;
	}

	/**
	 * get
	 * @param urlPath
	 * @return
	 */
	public static String doJsonGet(String urlPath) {

		CloseableHttpClient client = HttpClients.createDefault();
		HttpGet httpGet = new HttpGet();

		try {

			httpGet.setURI(new URI(urlPath));
			httpGet.addHeader("Content-Type", "application/json");
			httpGet.addHeader("charset","utf-8");
			CloseableHttpResponse resp = client.execute(httpGet);
			if (resp.getStatusLine().getStatusCode() == 200) {
				logger.debug("请求返回数据" + JSON.toJSONString(resp));
				return handleResponse(resp);
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("调用请求地址：" + urlPath + "失败！");
		}
		return null;
	}

	/**
	 * delete
	 * @param urlPath
	 * @return
	 */
	public static String doJsonDelete(String urlPath) {

		CloseableHttpClient client = HttpClients.createDefault();
		HttpDelete httpDelete = new HttpDelete();

		try {

			httpDelete.setURI(new URI(urlPath));
			httpDelete.addHeader("Content-Type", "application/json");
			CloseableHttpResponse resp = client.execute(httpDelete);
			if (resp.getStatusLine().getStatusCode() == 200) {
				logger.debug("请求返回数据" + JSON.toJSONString(resp));
				return handleResponse(resp);
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("调用请求地址：" + urlPath + "失败！");
		}
		return null;
	}

	private static String handleResponse(CloseableHttpResponse response) {
		int statusCode = response.getStatusLine().getStatusCode();
		if (200 == statusCode) {
			InputStream is;
			try {
				is = response.getEntity().getContent();
				return getStringFromInputStream(is);
			} catch (Exception e) {
				e.printStackTrace();
				return null;
			}
		} else {
			logger.error(String.valueOf(response));
			return null;
		}

	}

	private static String getStringFromInputStream(InputStream is) throws IOException {

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		byte[] buffer = new byte[1024];
		int length = -1;
		while ((length = is.read(buffer)) != -1) {
			baos.write(buffer, 0, length);
		}
		is.close();
		String state = baos.toString();
		baos.close();
		return state;
	}

//	public static void main(String[] args) {
////		String data = "{\n" +
////				"  \"carNo\": \"京NB6666\",\n" +
////				"  \"cardType\": 11,\n" +
////				"  \"inTime\": 1518144418965,\n" +
////				"  \"parkId\": 11,\n" +
////				"  \"type\": 1\n" +
////				"}";
////	String  result =	doJsonPost("http://localhost:8008/api/dataServer/onTheSpotCar",data);
////		System.err.println(JSON.toJSON(result));
//		String  data ="http://localhost:8008/api/dataServer/onTheSpotCar/京NB6666";
//		String  result = doJsonPut(data,"");
//		System.err.println("result:"+ JSON.toJSONString(result));
//	}
}
