package com.huaching.parking.local.parking_local_general.util.local;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 日期工具类
 *
 * @author xiajl
 */
public final class DateUtils {

    public static final String YEAR_MONTH_DAY_SECOND = "yyyy-MM-dd HH:mm:ss";
    public static final String YEAR_MONTH_DAY_SECOND2 = "yyyy/MM/dd HH:mm:ss";
    public static final String YEAR_MONTH_DAY_SECOND3 = "yyyy年MM月dd日 HH时mm分ss秒";
    public static final String YEAR_MONTH_DAY_SECOND4 = "yyyy-MM-dd HH:mm";
    public static final String YEAR_MONTH_DAY_SECOND5 = "yyyyMMddHHmmss";
    public static final String YEAR_MONTH_DAY = "yyyy-MM-dd";
    public static final String YEAR_MONTH_DAY2 = "yyyy年MM月dd日";
    public static final String YEAR_MONTH_DAY3 = "yyMMdd";
    public static final String YEAR_MONTH_DAY4 = "yyyyMMdd";
    public static final String YEAR_MONTH = "yyyy-MM";

    private static ThreadLocal<Map<String, SimpleDateFormat>> sThreadLocal = new ThreadLocal<>();

    /**
     * 获取解析特定时间格式的 SimpleDateFormat
     *
     * @param pattern 时间格式
     */
    private static SimpleDateFormat getDateFormat(String pattern) {
        Map<String, SimpleDateFormat> strDateFormatMap = sThreadLocal.get();

        if (strDateFormatMap == null) {
            strDateFormatMap = new HashMap<>();
        }

        SimpleDateFormat simpleDateFormat = strDateFormatMap.get(pattern);
        if (simpleDateFormat == null) {
            simpleDateFormat = new SimpleDateFormat(pattern, Locale.getDefault());
            strDateFormatMap.put(pattern, simpleDateFormat);
            sThreadLocal.set(strDateFormatMap);
        }

        return simpleDateFormat;
    }

    /**
     * 时间格式化
     */
    public static String formatDate(Date date, String pattern) {
        if (date == null || pattern.isEmpty()) {
            return null;
        }
        return getDateFormat(pattern).format(date);
    }

    /**
     * 时间格式化
     */
    public static Date coventDate(String time, String pattern) {
        if (time.isEmpty() || pattern.isEmpty()) {
            return null;
        }
        try {
            return getDateFormat(pattern).parse(time);
        } catch (ParseException e) {
            //e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据开始时间和结束时间得到中间相差day + "天" + hour + "小时" + minute+"分钟";
     *
     * @param open
     * @param close
     * @return
     */
    private static String subDate(Date open, Date close) {
        if (open == null) {
            open = new Date();
        }
        if (close == null) {
            close = new Date();
        }
        int minutes = (int) ((close.getTime() - open.getTime()) / (60 * 1000) + (((close.getTime() - open.getTime()) % (60 * 1000)) <= 0 ? 0 : 1));
        int day = minutes / (60 * 24);
        int hour = (minutes - day * 60 * 24) / 60;
        int minute = minutes - day * 60 * 24 - hour * 60;
        return day + "天" + hour + "小时" + minute + "分钟";
    }


    /**
     * 计算时间差(按分钟时不足一分钟的按照一分钟计算，不足一小时按照一小时计算)
     *
     * @param begin 开始时间
     * @param end   结束时间
     * @param unit  返回类型
     *              0 毫秒
     *              1 秒
     *              2 分
     *              3 小时
     *              4 天
     * @return
     */
    public static long subDate(Date begin, Date end, int unit) {
        long milis = end.getTime() - begin.getTime();
        long temp;
        switch (unit) {
            case 1:
                return milis / 1000;
            case 2:
                temp = milis / (1000 * 60);
                if (milis / (1000 * 60.0) - temp != 0) {
                    return temp + 1;
                }
                return temp;
            case 3:
                temp = milis / (1000 * 60 * 60);
                if (milis / (1000 * 60 * 60.0) - temp != 0) {
                    return temp + 1;
                }
                return temp;
            case 4:
                return milis / (1000 * 60 * 60 * 24);
            default:
                return milis;
        }
    }
    public static void main(String[] args) {
        Date begin = stringConvertDate("2020-04-23 17:18:23", 3);
        Date end = stringConvertDate("2020-04-24 00:17:24", 3);
//        System.out.println(getNextDayStart(begin,0).equals(getNextDayStart(end,0)));
        System.out.println(subDate(getNextDayStart(begin,0), getNextDayStart(end,0), 4));
    }


    /**
     * 获取明天开始时间戳
     */
    public static Date getNextDayStart(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取几分钟后的时间
     */
    public static Date getNextMinute(Date date, int minute) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MINUTE, minute);
        return calendar.getTime();
    }

    /**
     * 获取指定天数的开始时间戳
     */
    public static Date getNextDayStart(Date date, int day) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, day);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取指定天数的截止时间戳
     */
    public static Date getNextDayEnd(Date date, int day) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, day);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取每天的开始时间戳
     *
     * @param date
     * @return
     */
    public static Date getDayStart(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取每天的截至时间戳
     */
    public static Date getDayEnd(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 根据日期串判断当月第几周
     *
     * @param date
     * @return
     * @throws Exception
     */
    public static int getMonthWeekNum(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        //第几周
        return calendar.get(Calendar.WEEK_OF_MONTH);
    }


    /**
     * 获取当前日期的周一
     *
     * @param date
     * @return
     */
    public static Date getWeekBegin(Date date) {

        Calendar cal = Calendar.getInstance();
        // 设置一个星期的第一天，按中国的习惯一个星期的第一天是星期一
        cal.setFirstDayOfWeek(Calendar.MONDAY);
        // 获得当前日期是一个星期的第几天
        int dayWeek = cal.get(Calendar.DAY_OF_WEEK);
        if (dayWeek == 1) {
            dayWeek = 8;
        }
        // 根据日历的规则，给当前日期减去星期几与一个星期第一天的差值
        cal.add(Calendar.DATE, cal.getFirstDayOfWeek() - dayWeek);
        return cal.getTime();
    }

    public static Date getMonthDayEnd(Date startTime, int months) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startTime);
        calendar.add(Calendar.MONTH, months);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    public static String dateFormatToString(Date date,int flag){
        SimpleDateFormat sdf;
        switch (flag){
            case 0:
                sdf = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
                return sdf.format(date);
            case 1:
                sdf = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
                return sdf.format(date);
            case 2:
                sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                return sdf.format(date);
            case 3:
                sdf = new SimpleDateFormat("yyyy-MM-dd");
                return sdf.format(date);
            default:
                sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                return sdf.format(date);
        }
    }

    /**
     * String 转化 Date 0 yyyy-MM-dd 1 yyyy-MM-dd HH 2 yyyy-MM-dd HH:mm 3
     * yyyy-MM-dd HH:mm:ss 4 yyyy-MM 5 yyyy else yyyy-MM-dd
     *
     * @param time
     * @return
     */
    public static Date stringConvertDate(String time, int temp) {
        if (time == null || "".equals(time)) {
            return new Date();
        }

        SimpleDateFormat sdf = null;

        if (temp == 0) {
            sdf = new SimpleDateFormat("yyyy-MM-dd");
        } else if (temp == 1) {
            sdf = new SimpleDateFormat("yyyy-MM-dd HH");
        } else if (temp == 2) {
            sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        } else if (temp == 3) {
            sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        } else if (temp == 4) {
            sdf = new SimpleDateFormat("yyyy-MM");
        } else if (temp == 5) {
            sdf = new SimpleDateFormat("yyyy");
        } else {
            sdf = new SimpleDateFormat("yyyy-MM-dd");
        }

        try {
            return sdf.parse(time);
        } catch (ParseException e) {
            return new Date();
        }
    }

    public static Date getPassDate(int past) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_YEAR, calendar.get(Calendar.DAY_OF_YEAR) - past);
        return calendar.getTime();
    }

    /**
     * 给指定的日期加减指定的时间单位
     * @param date 日期
     * @param dateNum 加减的数量
     * @param type 时间单位
     * @return
     */
    public static Date calAddDate(Date date, int dateNum,int type) {
        try {
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.add(type, dateNum);
            return cal.getTime();
        } catch (Exception var5) {
            var5.printStackTrace();
            return null;
        }
    }

    /**
     * 判断当前时间是否在[startTime, endTime]区间，注意时间格式要一致
     *
     * @param nowTime   当前时间
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return
     * @author jqlin
     */
    public static boolean isEffectiveDate(Date nowTime, Date startTime, Date endTime) {
        if (nowTime.getTime() == startTime.getTime()
                || nowTime.getTime() == endTime.getTime()) {
            return true;
        }

        Calendar date = Calendar.getInstance();
        date.setTime(nowTime);

        Calendar begin = Calendar.getInstance();
        begin.setTime(startTime);

        Calendar end = Calendar.getInstance();
        end.setTime(endTime);

        if (date.after(begin) && date.before(end)) {
            return true;
        } else {
            return false;
        }
    }
    /*
     * 将时间戳转换为时间
     */
    public static String stampToDate(String s){
        String res;
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        long lt = new Long(s);
        Date date = new Date(lt);
        res = simpleDateFormat.format(date);
        return res;
    }
}
