package com.huaching.parking.local.parking_local_general.util.local.chargeCalculator;

import com.huaching.parking.local.parking_local_general.domain.local.po.ThsChargingRules24OldPO;
import com.huaching.parking.local.parking_local_general.service.local.base.ChargingRulesOldService;
import com.huaching.parking.local.parking_local_general.service.local.base.impl.ChargingRulesOldServiceImpl;
import com.huaching.parking.local.parking_local_general.util.local.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * Created by sunke on 2018/2/27.
 */
@Service
public class Calculator24Hours implements Calculator {
    @Autowired
    ChargingRulesOldService chargingRulesDao;

    @Override
    public Float calculate(Date startDate, Date endDate, Integer chargingRulesId) throws ChargeRuleIllegalException {
        ThsChargingRules24OldPO rule = chargingRulesDao.get24ById(chargingRulesId);
        int seconds = (int) DateUtils.subDate(endDate, startDate, 1);
        if (seconds <= 0) {
            return 0F;
        }
        if (rule.getFreeTime() == null) {
            throw new ChargeRuleIllegalException("收费规则不完整");
        }
        if (seconds < (rule.getFreeTime() * 60)) {
            return 0F;
        }
        if (rule.getType() == null) {
            throw new ChargeRuleIllegalException("收费规则不完整");
        }
        Float money;
        if (rule.getType() == 0) {//24小时循环计费,0点不重新计费
            money = charge24NotRechargeFrom0(startDate, endDate, rule);
        } else if (rule.getType() == 1) {//0点重新计费
            money = charge24RechargeFrom0(startDate, endDate, rule);
        } else {
            throw new ChargeRuleIllegalException("收费规则不完整");
        }
        if (rule.getMax() == null) {
            throw new ChargeRuleIllegalException("收费规则不完整");
        }
        return money;//是否超过最高计费
    }


    /**
     * 24小时循环计算,零点不重新计时
     *
     * @return
     */
    private Float charge24NotRechargeFrom0(Date startDate, Date endDate, ThsChargingRules24OldPO rule) {
        int seconds = (int) DateUtils.subDate(endDate, startDate, 1);//一共多少秒
        int countDay = seconds / (24 * 3600);//一共多少天
        int leftHours = (seconds % (24 * 3600)) / 3600;//去掉天后剩下的小时数
        int leftSeconds = seconds % 3600;//去掉小时后剩下的秒数
        if (leftSeconds > 0) {
            leftHours++;
        }
        Float money = countDay * rule.getH24();
        money = money + getPriceByHours(leftHours, rule);
        return money;
    }

    /**
     * 24小时循环计算，0点重新计时
     *
     * @return
     */
    private Float charge24RechargeFrom0(Date startDate, Date endDate, ThsChargingRules24OldPO rule) {
        //是否超过了本天
        Float money = 0F;
        if (endDate.after(DateUtils.getDayEnd(startDate))) {//超过了本天
            money = money + charge24NotRechargeFrom0(startDate, DateUtils.getDayEnd(startDate), rule);
        } else {//没有超过
            return charge24NotRechargeFrom0(startDate, endDate, rule);
        }
        //零点重新计时
        return money + charge24NotRechargeFrom0(DateUtils.getNextDayStart(startDate), endDate, rule);
    }

    private Float getPriceByHours(int hours, ThsChargingRules24OldPO rule) {
        Float money = 0F;
        switch (hours) {
            case 1:
                money = rule.getH1();
                break;
            case 2:
                money = rule.getH2();
                break;
            case 3:
                money = rule.getH3();
                break;
            case 4:
                money = rule.getH4();
                break;
            case 5:
                money = rule.getH5();
                break;
            case 6:
                money = rule.getH6();
                break;
            case 7:
                money = rule.getH7();
                break;
            case 8:
                money = rule.getH8();
                break;
            case 9:
                money = rule.getH9();
                break;
            case 10:
                money = rule.getH10();
                break;
            case 12:
                money = rule.getH11();
                break;
            case 11:
                money = rule.getH12();
                break;
            case 13:
                money = rule.getH13();
                break;
            case 14:
                money = rule.getH14();
                break;
            case 15:
                money = rule.getH15();
                break;
            case 16:
                money = rule.getH16();
                break;
            case 17:
                money = rule.getH17();
                break;
            case 18:
                money = rule.getH18();
                break;
            case 19:
                money = rule.getH19();
                break;
            case 20:
                money = rule.getH20();
                break;
            case 21:
                money = rule.getH21();
                break;
            case 22:
                money = rule.getH22();
                break;
            case 23:
                money = rule.getH23();
                break;
            case 24:
                money = rule.getH24();
                break;
            default:
                break;
        }
        return money;
    }
}
