package com.huaching.parking.local.parking_local_general.util.local.chargeCalculator;

import com.huaching.parking.local.parking_local_general.domain.local.po.ThsChargingRulesOldPO;
import com.huaching.parking.local.parking_local_general.service.local.base.ChargingRulesOldService;
import com.huaching.parking.local.parking_local_general.service.local.base.impl.ChargingRulesOldServiceImpl;
import com.huaching.parking.local.parking_local_general.util.local.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * Created by sunke on 2018/2/27.
 */
@Service
public class CalculatorNormal implements Calculator {
    @Autowired
    ChargingRulesOldService chargingRulesDao;

    @Override
    public Float calculate(Date startDate, Date endDate, Integer chargingRulesId) throws ChargeRuleIllegalException {
        if (startDate.getTime() >= endDate.getTime()) {
            return 0F;
        }
        ThsChargingRulesOldPO rule = chargingRulesDao.getById(chargingRulesId);
        if (rule == null) {
            throw new ChargeRuleIllegalException("收费规则不存在");
        }
        return calculate(startDate, endDate, rule);
    }


    public Float calculate(Date startDate, Date endDate, ThsChargingRulesOldPO rule) throws ChargeRuleIllegalException {
        Float freeTime = rule.getFreeTime();
        //如果没有超过免费时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);
        calendar.add(Calendar.MINUTE, freeTime.intValue());
        //开始计费时间为免费时间之后
        //设置游标开始计费时间
        Date cursorStartDate = calendar.getTime();
        if (cursorStartDate.getTime() >= endDate.getTime()) {
            return 0F;
        }
        calendar.setTime(startDate);
        cursorStartDate = calendar.getTime();
        //1：次；2：时段
        int chargeType = rule.getChargeType();
        //1:全天；2:白天夜间；
        int timeType = rule.getTimeType();

        //System.out.println("超过免费时间");
        //1.1按次收费（只收起步价） 统一收费
        if (chargeType == 1 && timeType == 1) {
            return chargeOneTimeWithOneCondition(rule);
        }
        //1.2.按次收费（只收起步价）  白/夜收费
        if (chargeType == 1 && timeType == 2) {
            return chargeOneTimeWithDayAndNight(cursorStartDate, rule);
        }
        //2.1按周期收费（起步价后每多少分钟收费一次） 统一收费
        if (chargeType == 2 && timeType == 1) {
            return chargeTimesWithOneCondition(cursorStartDate, endDate, rule);
        }
        //2.2按周期收费（起步价后每多少分钟收费一次） 白/夜收费
        if (chargeType == 2 && timeType == 2) {
            return chargeTimesWithDayAndNight(cursorStartDate, endDate, rule);
        }
        throw new ChargeRuleIllegalException("不正确的收费规则不完整");
    }


    /**
     * 单次收费  一个价
     *
     * @param rule
     * @return
     */
    private Float chargeOneTimeWithOneCondition(ThsChargingRulesOldPO rule) {
        Float dayTimesPrice = rule.getDayTimesPrice();
        return dayTimesPrice;
    }

    /**
     * 单次收费  分白天和晚上两个起步价
     *
     * @param cursorStartDate
     * @param rule
     * @return
     */
    private Float chargeOneTimeWithDayAndNight(Date cursorStartDate, ThsChargingRulesOldPO rule) {
        //默认白天开始时间小于晚上开始时间
        //白天开始时间
        String dayPointString = rule.getDayStartTime();
        //晚上开始时间
        String nightPointString = rule.getDayEndTime();
        //获取免费时长
        Float freeTime = rule.getFreeTime();
        Calendar calendar = Calendar.getInstance();
        Date startDate = cursorStartDate;
        calendar.setTime(startDate);
        calendar.add(Calendar.MINUTE, freeTime.intValue());
        //获取计算收费的时间
        //用此时时间来判断所在时段
        Date time = calendar.getTime();

        String[] dayPointHourAndMinute = dayPointString.split("[:：]");
        String[] nightPointHourAndMinute = nightPointString.split("[:：]");
        DayTime dayPoint = new DayTime(Integer.parseInt(dayPointHourAndMinute[0].trim()), Integer.parseInt(dayPointHourAndMinute[1].trim()));
        DayTime nightPoint = new DayTime(Integer.parseInt(nightPointHourAndMinute[0].trim()), Integer.parseInt(nightPointHourAndMinute[1].trim()));

        Float dayTimesPrice = rule.getDayTimesPrice();//白天起步价
        Float nightTimesPrice = rule.getNightTimesPrice();  //晚上起步价
        boolean isDay = isDay(dayPoint, nightPoint, time);
        //如果只有起步价的时间，计算之后直接返回
        if (isDay) {
            //System.out.println("白天短停收取起步价："+dayFirstPrice);
            return dayTimesPrice;
        } else {
            //System.out.println("夜间短停收取起步价："+dayFirstPrice);
            return nightTimesPrice;
        }
    }

    /**
     * 多次收费 一个价
     *
     * @param cursorStartDate
     * @param endDate
     * @param rule
     * @return
     */
    private Float chargeTimesWithOneCondition(Date cursorStartDate, Date endDate, ThsChargingRulesOldPO rule) {
        Calendar calendar = Calendar.getInstance();
        int circleTime = rule.getCycleTime().intValue();//周期
        int firstTime = Integer.parseInt(rule.getDayFirstTime());
        Float firstTimePrice = rule.getDayFirstTimePrice();
        Float RemainTimePrice = rule.getDayRemainTimePrice();

        Float total = 0F;//总计费
        calendar.setTime(cursorStartDate);
        calendar.add(Calendar.MINUTE, firstTime);
        cursorStartDate = calendar.getTime();
        //如果只有起步价的时间，计算之后直接返回
        if (cursorStartDate.getTime() >= endDate.getTime()) {
            return firstTimePrice;
        } else {
            total = firstTimePrice;
        }
        //开始计费时间为起步价时间之后
        //计算超过起步价的钱
        int seconds = (int) DateUtils.subDate(endDate, cursorStartDate, 1);
        //计算所有时间长
        int allSeconds = seconds + firstTime * 60;

        //长沙昌和购物中心停车场
        if (rule.getParkId() == 1821) {
            if (allSeconds < 6 * 3600) {
                total = 10f;
            } else if (allSeconds < 12 * 3600) {
                total = 15f;
            } else if (allSeconds < 24 * 3600) {
                total = 25f;
            } else {
                int dayCount = allSeconds / (24 * 3600);
                total = dayCount * 25f;
                int remainSeconds = allSeconds % (24 * 3600);
                if (remainSeconds < 3 * 3600) {
                    total = total + 5f;
                } else if (remainSeconds < 6 * 3600) {
                    total = total + 10f;
                } else if (remainSeconds < 12 * 3600) {
                    total = total + 15f;
                } else if (remainSeconds < 24 * 3600) {
                    total = total + 25f;
                }
            }
            return total;
        }

        /*if (rule.getParkId() == 1828000) {
            if(allSeconds<(24*3600)){
                int count = (seconds/(circleTime*60))+((seconds%(circleTime*60))>0?1:0);
                total = total + (count*RemainTimePrice);
                return total<6?total:6;
            }else{
                int dayCount = allSeconds/(24*3600);
                total = dayCount * rule.getAlldayMaximumAmount();
                int remainSeconds = allSeconds%(24*3600);
                int count  = (remainSeconds/(circleTime*60))+((remainSeconds%(circleTime*60))>0?1:0);
                float remainTotal = count*RemainTimePrice;
                remainTotal = remainTotal<rule.getAlldayMaximumAmount()?remainTotal:rule.getAlldayMaximumAmount();
                return total + remainTotal - 6;
            }
        }
*/
        if (allSeconds < (24 * 3600)) {
            int count = (seconds / (circleTime * 60)) + ((seconds % (circleTime * 60)) > 0 ? 1 : 0);
            total = total + (count * RemainTimePrice);
            return total < rule.getAlldayMaximumAmount() ? total : rule.getAlldayMaximumAmount();
        } else {
//
//            if(rule.getParkId()== || rule.getParkId()==114){
//                int overCount = ((allSeconds-(24*3600))/3600)+(((allSeconds-(24*3600))%3600)>0?1:0);
//                return rule.getAlldayMaximumAmount() + overCount*2.5f;
//            }else{
//                int dayCount = allSeconds/(24*3600);
//                total = dayCount * rule.getAlldayMaximumAmount();
//                int remainSeconds = allSeconds%(24*3600);
//                int count  = (remainSeconds/(circleTime*60))+((remainSeconds%(circleTime*60))>0?1:0);
//                float remainTotal = count*RemainTimePrice;
//                remainTotal = remainTotal<rule.getAlldayMaximumAmount()?remainTotal:rule.getAlldayMaximumAmount();
//                return total + remainTotal;
//            }


            int dayCount = allSeconds / (24 * 3600);
            total = dayCount * rule.getAlldayMaximumAmount();
            int remainSeconds = allSeconds % (24 * 3600);
            int count = (remainSeconds / (circleTime * 60)) + ((remainSeconds % (circleTime * 60)) > 0 ? 1 : 0);
            float remainTotal = count * RemainTimePrice;
            remainTotal = remainTotal < rule.getAlldayMaximumAmount() ? remainTotal : rule.getAlldayMaximumAmount();
            return total + remainTotal;

        }
    }

    /**
     * 多次收费 分白天晚上两个起步价和超时价
     *
     * @param cursorStartDate
     * @param endDate
     * @param rule
     * @return 如果用户停车时间刚好超过白天首时段收费周期，则不算白天首时段的费用
     */
    private Float chargeTimesWithDayAndNight(Date cursorStartDate, Date endDate, ThsChargingRulesOldPO rule) {
        Calendar calendar = Calendar.getInstance();
        //默认白天开始时间小于晚上开始时间
        //TODO排序两个时间
        //白天开始时间
        String dayPointString = rule.getDayStartTime();
        //晚上开始时间
        String nightPointString = rule.getDayEndTime();

        String[] dayPointHourAndMinute = dayPointString.split("[:：]");
        String[] nightPointHourAndMinute = nightPointString.split("[:：]");
        DayTime dayPoint = new DayTime(Integer.parseInt(dayPointHourAndMinute[0].trim()), Integer.parseInt(dayPointHourAndMinute[1].trim()));
        DayTime nightPoint = new DayTime(Integer.parseInt(nightPointHourAndMinute[0].trim()), Integer.parseInt(nightPointHourAndMinute[1].trim()));


        int circleTime = rule.getCycleTime().intValue();//周期
        int nightCircleTime = rule.getNightCycleTime().intValue();//晚上周期
        int dayFirstTime = Integer.parseInt(rule.getDayFirstTime());
        int nightFirstTime = Integer.parseInt(rule.getNightFirstTime());
        Float dayFirstPrice = rule.getDayFirstTimePrice();//白天起步价
        Float nightFirstPrice = rule.getNightFirstTimePrice();  //晚上起步价
        Float dayPrice = rule.getDayRemainTimePrice();
        Float nightPrice = rule.getNightRemainTimePrice();
        Float dayMaxMount = rule.getDayMaximumAmount();
        Float nightMaxMount = rule.getNightMaximumAmount();
        //获取全天最高收费
        Float alldayMaximumAmount = rule.getAlldayMaximumAmount();
        calendar.setTime(cursorStartDate);

        Float total = 0F;//总计费

        //获取免费时长
        Calendar calendar2 = Calendar.getInstance();
        Float freeTime = rule.getFreeTime();
        Date startDate = cursorStartDate;
        calendar2.setTime(startDate);
        calendar2.add(Calendar.MINUTE, freeTime.intValue());
        //获取计算收费的时间
        //用此时时间来判断所在时段
        Date time = calendar2.getTime();

        boolean isDay = isDay(dayPoint, nightPoint, time);

        //计算起步价
        if (isDay) {
            calendar.add(Calendar.MINUTE, dayFirstTime);
            cursorStartDate = calendar.getTime();
            if (cursorStartDate.getTime() >= endDate.getTime()) {
                return dayFirstPrice;
            }/*else{
                total=   dayFirstPrice;
            }*/
        } else {
            calendar.add(Calendar.MINUTE, nightFirstTime);
            cursorStartDate = calendar.getTime();
            if (cursorStartDate.getTime() >= endDate.getTime()) {
                return nightFirstPrice;
            }/*else{
                total=  nightFirstPrice;
            }*/
        }
        //开始计费时间为起步价时间之后
        //计算超过起步价的钱
        Date nextDate;

        int cycleCount = 0;

        while (true) {
            nextDate = getNextDate(dayPoint, nightPoint, cursorStartDate);
            isDay = isDay(dayPoint, nightPoint, cursorStartDate);
            //判断现阶段开始时间是否超过了结束时间
            if (cursorStartDate.getTime() >= endDate.getTime()) {
                break;
            }
            //判断结束时间是否超过了下一个时间点
            if (nextDate.before(endDate)) {
                //计算该时间段的价钱
                int seconds = (int) DateUtils.subDate(nextDate, cursorStartDate, 1);

                //System.out.println("超时时间："+minutes);
                //System.out.println("超时单位："+count);
                if (isDay) {
                    int count = (seconds / (circleTime * 60)) + ((seconds % (circleTime * 60)) > 0 ? 1 : 0);
                    System.out.println("白天超时时段收费：" + count * dayPrice);
                    float curPrice;
                    if (cycleCount == 0) {
                        curPrice = count * dayPrice + dayFirstPrice;
                    } else {
                        curPrice = count * dayPrice;
                    }
                    if (cycleCount == 0) {
                        //当前时间是否在白天时段内
                        if (this.isInDate(cursorStartDate, dayPointString, nightPointString)) {
                            dayMaxMount = dayMaxMount - total;
                        }
                    }
                    if (curPrice >= dayMaxMount) {
                        curPrice = dayMaxMount;
                    }
                    total = total + curPrice;

                    calendar.add(Calendar.MINUTE, count * circleTime);
                } else {
                    int count = (seconds / (nightCircleTime * 60)) + ((seconds % (nightCircleTime * 60)) > 0 ? 1 : 0);
                    System.out.println("夜间超时时段收费：" + count * nightPrice);
                    float curPrice;
                    if (cycleCount == 0) {
                        curPrice = count * nightPrice + nightFirstPrice;
                    } else {
                        curPrice = count * nightPrice;
                    }
                    if (cycleCount == 0) {
                        if (this.isInDate(cursorStartDate, dayPointString, nightPointString)) {
                            nightMaxMount = nightMaxMount - total;
                        }
                    }
                    if (curPrice >= nightMaxMount) {
                        curPrice = nightMaxMount;
                    }
                    total = total + curPrice;

                    calendar.add(Calendar.MINUTE, count * nightCircleTime);
                }
//                calendar.setTime(cursorStartDate);
//                calendar.add(Calendar.MINUTE,count*circleTime);
                cursorStartDate = calendar.getTime();
            } else {
                //计算最终一个时间段的价钱
                int seconds = (int) DateUtils.subDate(endDate, cursorStartDate, 1);
                //System.out.println("超时时间："+minutes);
                //System.out.println("超时单位："+count);
                if (isDay) {
                    int count = (seconds / (circleTime * 60)) + ((seconds % (circleTime * 60)) > 0 ? 1 : 0);
                    System.out.println("白天超时时段收费（最终时段）：" + count * dayPrice);
                    float curPrice;
                    if (cycleCount == 0) {
                        curPrice = count * dayPrice + dayFirstPrice;
                    } else {
                        curPrice = count * dayPrice;
                    }
                    if (cycleCount == 0) {
                        if (this.isInDate(cursorStartDate, dayPointString, nightPointString)) {
                            dayMaxMount = dayMaxMount - total;
                        }
                    }
                    if (curPrice >= dayMaxMount) {
                        curPrice = dayMaxMount;
                    }
                    total = total + curPrice;

                } else {
                    int count = (seconds / (nightCircleTime * 60)) + ((seconds % (nightCircleTime * 60)) > 0 ? 1 : 0);
                    System.out.println("夜间超时时段收费（最终时段）：" + count * nightPrice);
                    count = (seconds / (nightCircleTime * 60)) + ((seconds % (nightCircleTime * 60)) > 0 ? 1 : 0);
                    float curPrice;
                    if (cycleCount == 0) {
                        curPrice = count * nightPrice + nightFirstPrice;
                    } else {
                        curPrice = count * nightPrice;
                    }
                    if (cycleCount == 0) {
                        //同时段的最高收费做比较
                        if (this.isInDate(cursorStartDate, dayPointString, nightPointString)) {
                            nightMaxMount = nightMaxMount - total;
                        }
                    }
                    if (curPrice >= nightMaxMount) {
                        curPrice = nightMaxMount;
                    }
                    total = total + curPrice;

                }
                break;
            }
            cycleCount++;
        }
        //此处用总计费的价格与全天最高收费比较，如果大于等于全天最高收费，则全天最高收费
        //停车时间没有超过一天
        if (0 == cycleCount || 1 == cycleCount) {
            if (alldayMaximumAmount <= total) {
                return alldayMaximumAmount;
            }
        } else if (2 == cycleCount) {
            Calendar cal = Calendar.getInstance();
            cal.setTime(time);
            cal.add(Calendar.DATE, 1);//增加一天
            Date calDate = cal.getTime();
            if (calDate.after(nextDate)) {
                if (alldayMaximumAmount <= total) {
                    return alldayMaximumAmount;
                }
            }
            //此处判断车辆的进出时间是否构成一天(按24小时计算)
            if (Is24Hours(startDate, endDate)) {
                if (alldayMaximumAmount <= total) {
                    return alldayMaximumAmount;
                }
            }
        }
        return total;
    }

    /**
     * 获取下一个时间点
     *
     * @param dayPoint
     * @param nightPoint
     * @param startDate
     * @return
     */
    private Date getNextDate(DayTime dayPoint, DayTime nightPoint, Date startDate) {
        Calendar calendar = Calendar.getInstance();
        DayTime startPoint = new DayTime(startDate);
        calendar.setTime(startDate);
        //下一个时间点
        //随便一个值
        DayTime nextPoint = dayPoint;
        boolean ifTomorrow = false;
        //本计费是白天还是晚上
        //获取下一个计费规则切换的时间节点
        if (startPoint.lessThan(nightPoint) && startPoint.lessThan(dayPoint)) {
            if (nightPoint.lessThan(dayPoint)) {
                nextPoint = nightPoint;
            } else {
                nextPoint = dayPoint;
            }
            ifTomorrow = false;
        }
        if (startPoint.greaterOrEqual(nightPoint) && startPoint.lessThan(dayPoint)) {
            nextPoint = dayPoint;
            ifTomorrow = false;
        }
        if (startPoint.greaterOrEqual(dayPoint) && startPoint.lessThan(nightPoint)) {
            nextPoint = nightPoint;
            ifTomorrow = false;
        }

        if (startPoint.greaterOrEqual(nightPoint) && startPoint.greaterOrEqual(dayPoint)) {
            if (nightPoint.lessThan(dayPoint)) {
                nextPoint = nightPoint;
            } else {
                nextPoint = dayPoint;
            }
            ifTomorrow = true;
        }
        //得到下一个时间点
        calendar.setTime(startDate);
        if (ifTomorrow) {
            calendar.add(Calendar.DATE, 1);
            calendar.set(Calendar.HOUR_OF_DAY, nextPoint.hour);
            calendar.set(Calendar.MINUTE, nextPoint.minute);
        } else {
            calendar.set(Calendar.HOUR_OF_DAY, nextPoint.hour);
            calendar.set(Calendar.MINUTE, nextPoint.minute);
        }
        //得到下一个时间点
        return calendar.getTime();
    }

    /**
     * 判断本次计算是白天时段还是晚上时段
     *
     * @param dayPoint
     * @param nightPoint
     * @param startDate
     * @return
     */
    private boolean isDay(DayTime dayPoint, DayTime nightPoint, Date startDate) {
        Calendar calendar = Calendar.getInstance();
        DayTime startPoint = new DayTime(startDate);
        calendar.setTime(startDate);
        boolean isDay = false;
        //获取下一个计费规则切换的时间节点
        if (startPoint.lessThan(nightPoint) && startPoint.lessThan(dayPoint)) {
            if (nightPoint.lessThan(dayPoint)) {
                isDay = true;
            } else {
                isDay = false;
            }
        }
        if (startPoint.greaterOrEqual(nightPoint) && startPoint.lessThan(dayPoint)) {
            isDay = false;
        }
        if (startPoint.greaterOrEqual(dayPoint) && startPoint.lessThan(nightPoint)) {
            isDay = true;
        }

        if (startPoint.greaterOrEqual(nightPoint) && startPoint.greaterOrEqual(dayPoint)) {
            if (nightPoint.lessThan(dayPoint)) {
                isDay = true;
            } else {
                isDay = false;
            }
        }
        return isDay;
    }

    private class DayTime {
        public DayTime(int hour, int minute) {
            this.hour = hour;
            this.minute = minute;
        }

        public DayTime(Date date) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            this.hour = calendar.get(Calendar.HOUR_OF_DAY);
            this.minute = calendar.get(Calendar.MINUTE);
        }

        int hour;
        int minute;

        public boolean greaterOrEqual(DayTime nextDayTime) {
            if (this.hour > nextDayTime.hour) {
                return true;
            }
            if (this.hour < nextDayTime.hour) {
                return false;
            }
            if (this.minute >= nextDayTime.minute) {
                return true;
            }
            return false;
        }

        public boolean lessThan(DayTime nextDayTime) {
            if (this.greaterOrEqual(nextDayTime)) {
                return false;
            }
            return true;
        }

        @Override
        public String toString() {
            return hour + ":" + minute;
        }
    }

    /**
     * 　　* 判断时间是否在时间段内 *
     * <p>
     * 　　* @param date
     * <p>
     * 　　* 当前时间 yyyy-MM-dd HH:mm:ss
     * <p>
     * 　　* @param strDateBegin
     * <p>
     * 　　* 开始时间 00:00:00
     * <p>
     * 　　* @param strDateEnd
     * <p>
     * 　　* 结束时间 00:05:00
     * <p>
     * 　　* @return
     * <p>
     */
    public boolean isInDate(Date date, String strDateBegin, String strDateEnd) {

        if (strDateBegin.length() < 8) {
            strDateBegin += ":00";
        } else if (strDateBegin.length() < 6) {
            strDateBegin += ":00:00";
        }
        if (strDateEnd.length() < 8) {
            strDateEnd += ":00";
        } else if (strDateEnd.length() < 6) {
            strDateEnd += ":00:00";
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        String strDate = sdf.format(date);
        // 截取当前时间时分秒
        int strDateH = Integer.parseInt(strDate.substring(11, 13));

        int strDateM = Integer.parseInt(strDate.substring(14, 16));

        int strDateS = Integer.parseInt(strDate.substring(17, 19));

        // 截取开始时间时分秒

        int strDateBeginH = Integer.parseInt(strDateBegin.substring(0, 2));

        int strDateBeginM = Integer.parseInt(strDateBegin.substring(3, 5));

        int strDateBeginS = Integer.parseInt(strDateBegin.substring(6, 8));

        // 截取结束时间时分秒

        int strDateEndH = Integer.parseInt(strDateEnd.substring(0, 2));

        int strDateEndM = Integer.parseInt(strDateEnd.substring(3, 5));

        int strDateEndS = Integer.parseInt(strDateEnd.substring(6, 8));

        if ((strDateH >= strDateBeginH && strDateH <= strDateEndH)) {
            // 当前时间小时数在开始时间和结束时间小时数之间
            if (strDateH > strDateBeginH && strDateH < strDateEndH) {
                return true;
                // 当前时间小时数等于开始时间小时数，分钟数在开始和结束之间
            } else if (strDateH == strDateBeginH && strDateM >= strDateBeginM && strDateM <= strDateEndM) {
                return true;
                // 当前时间小时数等于开始时间小时数，分钟数等于开始时间分钟数，秒数在开始和结束之间
            } else if (strDateH == strDateBeginH && strDateM == strDateBeginM && strDateS >= strDateBeginS && strDateS <= strDateEndS) {
                return true;
                // 当前时间小时数大等于开始时间小时数，等于结束时间小时数，分钟数小等于结束时间分钟数
            } else if (strDateH >= strDateBeginH && strDateH == strDateEndH && strDateM <= strDateEndM) {
                return true;
                // 当前时间小时数大等于开始时间小时数，等于结束时间小时数，分钟数等于结束时间分钟数，秒数小等于结束时间秒数
            } else if (strDateH >= strDateBeginH && strDateH == strDateEndH && strDateM == strDateEndM && strDateS <= strDateEndS) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    //判断是否超过24小时
    public boolean Is24Hours(Date date1, Date date2) {
        Date start = date1;
        Date end = date2;
        long cha = end.getTime() - start.getTime();
        double result = cha * 1.0 / (1000 * 60 * 60);
        if (result <= 24) {
            //System.out.println("可用");
            return true;
        } else {
            //System.out.println("已过期");
            return false;
        }
    }


    public static void main(String[] args) {
        try {
            ThsChargingRulesOldPO rules = new ThsChargingRulesOldPO();
            rules.setFreeTime(30F);
            rules.setDayStartTime("08:00");
            rules.setDayEndTime("17:30");
            rules.setNightStartTime("17:30");
            rules.setNightEndTime("08:00");
            rules.setDayFirstTime("75");
            //30分钟一个周期
            rules.setCycleTime(60F);
            rules.setNightCycleTime(60f);
            rules.setNightFirstTime("75");
            //白天起步价8
            rules.setDayFirstTimePrice(4F);
            //白天超时价6
            rules.setDayRemainTimePrice(2f);
            //晚上起步价6
            rules.setNightFirstTimePrice(0F);
            //晚上超时价4
            rules.setNightRemainTimePrice(0f);
            rules.setDayMaximumAmount(20f);
            rules.setNightMaximumAmount(0f);
            rules.setAlldayMaximumAmount(20f);

            //白天单次价  -----也同时可以是全天单次价
            rules.setDayTimesPrice(5F);
            //夜间单次价
            rules.setNightTimesPrice(5F);

            //1：次；2：时段
            //int chargeType = rule.getChargeType();
            //1:全天；2:白天夜间；
            // int timeType = rule.getTimeType();

            rules.setChargeType(2);
            rules.setTimeType(2);
            rules.setParkId(1996);

            Calendar calendar = Calendar.getInstance();
            //2017-7-6 11:30:00
            calendar.set(2019, 11, 24, 12, 15, 50);
            Date startDate = calendar.getTime();
            System.out.println("开始时间：" + calendar.get(Calendar.DAY_OF_MONTH) + "-" + calendar.get(Calendar.HOUR_OF_DAY) + ":" + calendar.get(Calendar.MINUTE));
            //2017-7-6 13:30:00
            /*calendar.set(2017,7,7,3,15,0);
            Date endDate = calendar.getTime();
            Float total = null;
            try {
                total = ChargeCalculator.calculate(startDate,endDate,rules);
            } catch (ChargeRuleIllegalException e) {
                e.printStackTrace();
            }
            System.out.println(total);*/

            CalculatorNormal calculatorNormal = new CalculatorNormal();
            float t = calculatorNormal.calculate(startDate, calendar.getTime(), rules);
            System.out.println(t + "===");
            int count = 100;
            while (true) {
                calendar.add(Calendar.MINUTE, 30);
                Date endDate = calendar.getTime();
                System.out.println("现在时间：" + calendar.get(Calendar.DAY_OF_MONTH) + "-" + calendar.get(Calendar.HOUR_OF_DAY) + ":" + calendar.get(Calendar.MINUTE) + ":" + calendar.get(Calendar.SECOND) + "     ");
                Float total = null;
                try {
                    total = calculatorNormal.calculate(startDate, endDate, rules);
                } catch (ChargeRuleIllegalException e) {
                    e.printStackTrace();
                }
                System.out.println(total);
                count--;
                if (count <= 0) {
                    break;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
