package com.huaching.parking.local.parking_local_general.util.local.chargeCalculator;

import com.huaching.parking.local.parking_local_general.domain.local.po.ThsChargingRules24OldPO;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsChargingRulesOldPO;
import com.huaching.parking.local.parking_local_general.service.local.base.ChargingRulesOldService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * Created by sunke on 2018/2/27.
 */
@Service
public class ChargeCalculatorUtil {
    @Autowired
    CalculatorNormal calculatorNormal;
    @Autowired
    Calculator24Hours calculator24Hours;
    @Autowired
    CalculatorFree calculatorFree;
    @Autowired
    CalculatorCarSpace calculatorCarSpace;
    @Autowired
    ChargingRulesOldService chargingRulesService;

    /**
     * 获取收费信息
     *
     * @param parkId     停车的车场
     * @param chargeType 收费规则类型
     * @return 收费信息实体
     * @throws Exception
     */
    public Integer getChargeInfo(Date begin, Date end, Integer parkId, byte chargeType, boolean free) throws Exception {
        ChargeInfo info = new ChargeInfo();
        //换算成单位分
        //选择Calculator

        ChargeInfo chargeInfo = calculate(begin, end, parkId, chargeType, free);
        return chargeInfo.getChargeTotal();
    }

    /**
     * 计算收费总入口
     *
     * @param begin 开始时间
     * @param end   结束时间
     * @return
     * @throws Exception
     */
    private ChargeInfo calculate(Date begin, Date end, Integer parkId, byte chargeType, boolean free) throws Exception {

        //加载默认收费规则
        Byte parkChargingRulesType = chargeType;
        Integer parkChargingRulesId = null;
        //如果是免费车场
        if (free) {
            ChargeInfo calculate = new ChargeInfo();
            calculate.setChargeTotal(0);
            calculate.setMemberReceivableAmount(0);
            calculate.setPayedValue(0);
            calculate.setDiscountRatio(0);
            //免费
            return calculate;
        }

        if (parkChargingRulesType == 0) {
            ThsChargingRulesOldPO chargingRules = chargingRulesService.getDefaultByParkId(parkId);
            parkChargingRulesId = chargingRules == null ? null : chargingRules.getId();
        } else if (parkChargingRulesType == 1 || parkChargingRulesType == 2) {
            ThsChargingRules24OldPO chargingRules24 = chargingRulesService.getDefault24ByParkId(parkId);
            parkChargingRulesId = chargingRules24 == null ? null : chargingRules24.getId();
        }

        if (parkChargingRulesId == null) {
            throw new Calculator.ChargeRuleIllegalException("收费规则不完整");
        }

        //车场基本收费规则
        //将基本的默认收费规则存储到bean中
        ChargingRulesEntry parkChargingRulesEntry = new ChargingRulesEntry(parkChargingRulesType, parkChargingRulesId, null, null);
        //计算
        ChargeInfo calculate = calculate(begin, end, parkChargingRulesEntry);
        Float charge = calculate.getTotal();
        calculate.setChargeTotal(charge.intValue());
        return calculate;
        //}


    }

    /**
     * 计算卡券类型的收费
     *
     * @param start
     * @param end
     * @param parkChargingRulesEntry 车场收费规则实体
     * @return
     * @throws Exception
     */
    private ChargeInfo calculate(Date start, Date end, ChargingRulesEntry parkChargingRulesEntry) throws Exception {
        ChargeInfo chargeInfo = new ChargeInfo();

        //车场收费规则计算器
        Calculator parkCalculator = getCalculator(parkChargingRulesEntry.getChargingRulesType());
        //车场收费规则id
        Integer parkChargingRulesId = parkChargingRulesEntry.getChargingRulesId();
        //此处计算停车费用时，不区分会员与非会员
        Float calculate = parkCalculator.calculate(start, end, parkChargingRulesId);
        chargeInfo.setTotal(calculate);

        return chargeInfo;
    }

    private Calculator getCalculator(Byte chargingRulesType) throws Exception {
        if (chargingRulesType == null) {
            throw new Exception("收费规则不完整");
        }
        switch (chargingRulesType) {
            case 0:
                return calculatorNormal;
            case 1:
                return calculator24Hours;
            case 2:
                return calculatorCarSpace;
            case 3:
                return calculatorFree;
        }
        throw new Exception("收费规则不完整");
    }

    /**
     * 用于存储
     * <p>
     * 收费规则类型
     * 收费规则id
     * 收费规则的开始时间
     * 收费规则的结束时间
     */
    class ChargingRulesEntry implements Comparable<ChargingRulesEntry> {
        private Byte chargingRulesType;
        private Integer chargingRulesId;
        private Date startDate;
        private Date endDate;

        public Byte getChargingRulesType() {
            return chargingRulesType;
        }

        public Integer getChargingRulesId() {
            return chargingRulesId;
        }

        public Date getStartDate() {
            return startDate;
        }

        public Date getEndDate() {
            return endDate;
        }

        ChargingRulesEntry(Byte chargingRulesType, Integer chargingRulesId, Date startDate, Date endDate) {
            this.chargingRulesType = chargingRulesType;
            this.chargingRulesId = chargingRulesId;
            this.startDate = startDate;
            this.endDate = endDate;
        }

        //用于按时间排序
        @Override
        public int compareTo(ChargingRulesEntry o) {
            return (this.getStartDate().getTime() < o.getStartDate().getTime()) ? -1 : ((this.getStartDate().getTime() == o.getStartDate().getTime()) ? 0 : 1);
        }
    }
}
